@@ -60,11 +60,11 @@
  */
 
 #include <stdarg.h>
-
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
+#include <stdlib.h>
 #ifdef __bsdi__
 #include <sys/malloc.h>
 #else
@@ -73,7 +73,9 @@
 #include <time.h>
 #include <sys/types.h>
 #include <time.h>
-#include "../libmc/mc_defs.h"
+
+#include "HTMLmiscdefs.h"
+#include "HTMLparse.h"
 #include "HTMLP.h"
 #include "HTMLPutil.h"
 
@@ -85,10 +87,6 @@
 #define CR '\015'
 #define LF '\012'
 
-#ifndef DISABLE_TRACE
-extern int htmlwTrace;
-#endif
-
 #define USLETTER 	0
 #define A4 		1
 
@@ -180,19 +178,10 @@ PAGE_DIMENS_T	us_letter_page_dimens = {
     0.9 * INCH
 };
 
-
-/* Globals to get button value in print dialog -- swp/AF */
-
-int HTML_Print_Headers       = 1;	/* Flag whether page headers enabled */
-int HTML_Print_Footers     = 1;	/* Flag whether footnote printing enabled */
-
 /* Paper format (currently either A4 or letter).  This should be generalized. */
 
 int HTML_Print_Paper_Size_A4 = DEFAULT_PAGE_SIZE;
 
-extern int installed_colormap;
-extern Colormap installed_cmap;
-
 /*
  * GetDpi - return Dots-per-inch of the screen
  *
@@ -235,10 +224,8 @@ static int PSprintf(format, va_alist)
 
     if (PS_size - PS_len < 1024) {
 	PS_size += 1024;
-	if ((s = (char *) realloc(PS_string, PS_size)) == NULL) {
-		fprintf(stderr, "PSprintf malloc failed\n");
-		return(EOF);
-	}
+	s = (char *) realloc(PS_string, PS_size);
+	CHECK_OUT_OF_MEM(s);
 	PS_string = s;
     }
     va_start(ap);
@@ -260,16 +247,14 @@ static int PSprintf(char *format, ...)
   
     if (PS_size - PS_len < 1024) {
         PS_size += 1024;
-        if ((s = (char *) realloc(PS_string, PS_size)) == NULL) {
-		fprintf(stderr, "PSprintf malloc failed\n");
-                return(EOF);
-        }
+        s = (char *) realloc(PS_string, PS_size);
+	CHECK_OUT_OF_MEM(s);
         PS_string = s;
     }           
     len = vsprintf(PS_string+PS_len, format, args);
     /* this is a hack to make it work on systems were vsprintf(s,.)
      * returns s, instead of the len. */
-    if (len != EOF && len != NULL)
+    if (len != EOF && len != 0)
         PS_len += strlen(PS_string+PS_len);
     va_end(args);
     return(len);
@@ -311,154 +296,171 @@ static int PShex(unsigned char val, int flush)
  */
 static void PSfont(HTMLWidget hw, XFontStruct *font, int fontfamily) 
 {
-    PS_fontstyle fn;
-    int style, size;
-    int fs;
-
-    static PS_fontstyle fontstyle[17] = {
-	RF, IF, BF, FF, BF, BF, BF, BF, BF,
-	BF, IF, FF, FF, FB, FI, FB, FI
-    };
-    static char fnchar[6][3] = {"RF", "BF", "IF", "FF", "FB", "FI"};
-
-    /* fontsizes as set in gui.c and in HTML.c (listing font is only
-     * defined in HTML.c)
-     */
-    static int fontsizes[4][3][17] = {
-	/* times font sizes */
-	{
-	    {14, 14, 14, 14, 18, 17, 14, 12, 10, 8, 14, 12, 12, 14, 14, 12, 12},
-	    {17, 17, 17, 17, 24, 18, 17, 14, 12, 10, 17, 14, 12, 17, 17, 14, 14},
-	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
-	},
-	/* helvetica sizes */
-	{
-	    {14, 14, 14, 14, 18, 17, 14, 12, 10, 8, 14, 12, 12, 14, 14, 12, 12},
-	    {17, 17, 17, 17, 24, 18, 17, 14, 12, 10, 17, 14, 12, 17, 17, 14, 14},
-	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
-	},
-	/* new century schoolbook sizes */
-	{
-	    {14, 14, 14, 14, 18, 17, 14, 12, 10, 8, 14, 12, 12, 14, 14, 12, 12},
-	    {18, 18, 18, 18, 24, 18, 17, 14, 12, 10, 18, 14, 12, 18, 18, 14, 14},
-	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
-	},
-	/* lucida sizes */
-	{
-	    {14, 14, 14, 14, 18, 17, 14, 12, 11, 10, 14, 12, 12, 14, 14, 12, 12},
-	    {17, 17, 17, 17, 24, 18, 17, 14, 12, 10, 17, 14, 12, 17, 17, 14, 14},
-	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
-	}
-    };
-
-    /* next is for each fontfamily the ascent value as given by the 
-     * medium sized bold x-font (the regular font has the same
-     * ascent value for both the medium and the large size Century
-     * font).
-     * it is use in the check for the fontsize (small, medium, large)
-     */
-    static int medium_fontascent[4] = {
-	14, 14, 16, 15
-    };
-
-    /* for each fontfamily, for each fontsize, and for each font style
-     * give the ascent value, so the output from Postscript is correct.
-     * If the value is given between parenthesis, then it is different
-     * from the value as stored in the x-font.
-     * Note that this is a fix, and need to be changed, if the browser
-     * is fixed (in the current version 1.2 the baseline of various fonts
-     * is not aligned very well).
-     */
-    static int fontascent[4][3][17] = {
-	/*rg, itl, bld, fix,  h1,  h2,  h3,  h4,  h5,  h6,
-	  add, pla, lis, fixbold, fixital, plabold, plaital, */
-	/* times */
-	{
-	    {12, 11, 12, 10, 15, 14, 12, 10, 8, 6, 11, 9, 10, 10, 10, 9, 9},
-	    {13, 13, 14, 12, 20, 15, 14, 12, 10, 8, 13, 10, 10, 12, 12, 10, 10},
-	    {16, 15, 15, 13, 21, 20, 15, 15, 14, 12, 15, 13, 10, 13, 13, 13, 13}
-	},
-	/* helvetica */
-	{
-	    {12, 12, 12, 10, 15, 14, 12, 10, 9, 7, 12, 9, 10, 10, 10, 9, 9},
-	    {14, 14, 14, 12, 22, 15, 14, 12, 10, 9, 14, 10, 10, 12, 12, 10, 10},
-	    {16, 16, 16, 13, 22, 22, 16, 15, 14, 12, 16, 13, 10, 13, 13, 13, 13}
-	},
-	/* new century schoolbook */
-	{
-	    {12, 12, 13, 10, 16, 14, 13, 10, 9, 7, 12, 9, 10, 10, 10, 9, 9},
-	    {16, 14, 16, 13, 22, 16, 14, 13, 10, 9, 14, 10, 10, 13, 13, 10, 10},
-	    {17, 16, 17, 13, 22, 22, 17, 16, 14, 13, 16, 13, 10, 13, 13, 13, 13}
-	},
-	/* lucida bright */
-	{
-	    {11, 11, 11, 11, 15, 14, 11, 10, 9, 7, 11, 9, 10, 11, 10, 9, 9},
-	    {14, 15, 14, 13, 20, 15, 14, 11, 10, 7, 15, 11, 10, 13, 13, 11, 10},
-	    {17, 17, 17, 16, 21, 20, 17, 15, 14, 11, 17, 14, 10, 16, 13, 14, 13}
-	}
-    };
-
-    /* NULL case - reflush old font or the builtin default: */
-    if ((hw == NULL) || (font == NULL)) {
-	if (PS_oldfs != 0)
-	    PSprintf( "%2s %d SF\n", fnchar[PS_oldfn], PS_oldfs);
-	return;
-    }
-    /* added the next line in case xmosaic version 199.4 has more fonts */
-    style = 3;
-    if (font == hw->html.font) {
-	style = 0;
-    } else if (font == hw->html.italic_font) {
-	style = 1;
-    } else if (font == hw->html.bold_font) {
-	style = 2;
-    } else if (font == hw->html.fixed_font) {
-	style = 3;
-    } else if (font == hw->html.header1_font) {
-	style = 4;
-    } else if (font == hw->html.header2_font) {
-	style = 5;
-    } else if (font == hw->html.header3_font) {
-	style = 6;
-    } else if (font == hw->html.header4_font) {
-	style = 7;
-    } else if (font == hw->html.header5_font) {
-	style = 8;
-    } else if (font == hw->html.header6_font) {
-	style = 9;
-    } else if (font == hw->html.address_font) {
-	style = 10;
-    } else if (font == hw->html.plain_font) {
-	style = 11;
-    } else if (font == hw->html.listing_font) {
-	style = 12;
-    } else if (font == hw->html.fixedbold_font) {
-	style = 13;
-    } else if (font == hw->html.fixeditalic_font) {
-	style = 14;
-    } else if (font == hw->html.plainbold_font) {
-	style = 15;
-    } else if (font == hw->html.plainitalic_font) {
-	style = 16;
-    }
-
-    /* check size, by looking at the size of the regular font */
-    size = 1;
-    if (hw->html.bold_font->ascent > medium_fontascent[fontfamily]) {
-	/* large font */
-	size = 2;
-    } else if (hw->html.bold_font->ascent < medium_fontascent[fontfamily]) {
-	/* small font */
-	size = 0;
-    }
-    fn = fontstyle[style];
-    fs = fontsizes[fontfamily][size][style];
-    PS_fontascent = fontascent[fontfamily][size][style];
-
-    if (fn != PS_oldfn || fs != PS_oldfs) {
-	PSprintf( "%2s %d SF\n", fnchar[fn], fs);
-	PS_oldfn=fn, PS_oldfs=fs;
-    }
+/*########################################
+CHANGE ALL THIS CODE BECAUSE OF REMANAGEMENT OF FONT IN MMOSAIC
+TRANSLATE A XLFD FONT TO A POSTSCRIPT FONT
+ *
+ *    PS_fontstyle fn;
+ *    int style, size;
+ *    int fs;
+ *
+ *    static PS_fontstyle fontstyle[17] = {
+ *	RF, IF, BF, FF, BF, BF, BF, BF, BF,
+ *	BF, IF, FF, FF, FB, FI, FB, FI
+ *   };
+ *  static char fnchar[6][3] = {"RF", "BF", "IF", "FF", "FB", "FI"};
+ */
+/* fontsizes as set in gui.c and in HTML.c (listing font is only
+ * defined in HTML.c)
+ */
+/*    static int fontsizes[4][3][17] = { */
+/* times font sizes */
+/*	{
+ *	    {14, 14, 14, 14, 18, 17, 14, 12, 10, 8, 14, 12, 12, 14, 14, 12, 12},
+ *	    {17, 17, 17, 17, 24, 18, 17, 14, 12, 10, 17, 14, 12, 17, 17, 14, 14},
+ *	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
+ *	},
+*/
+/* helvetica sizes */
+/*	{
+ *	    {14, 14, 14, 14, 18, 17, 14, 12, 10, 8, 14, 12, 12, 14, 14, 12, 12},
+ *	    {17, 17, 17, 17, 24, 18, 17, 14, 12, 10, 17, 14, 12, 17, 17, 14, 14},
+ *	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
+ *	},
+*/
+/* new century schoolbook sizes */
+/*	{
+ *	    {14, 14, 14, 14, 18, 17, 14, 12, 10, 8, 14, 12, 12, 14, 14, 12, 12},
+ *	    {18, 18, 18, 18, 24, 18, 17, 14, 12, 10, 18, 14, 12, 18, 18, 14, 14},
+ *	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
+ *	},
+*/
+/* lucida sizes */
+/*	{
+ *	    {14, 14, 14, 14, 18, 17, 14, 12, 11, 10, 14, 12, 12, 14, 14, 12, 12},
+ *	    {17, 17, 17, 17, 24, 18, 17, 14, 12, 10, 17, 14, 12, 17, 17, 14, 14},
+ *	    {20, 20, 20, 20, 25, 24, 20, 18, 17, 14, 20, 18, 12, 20, 20, 18, 18}
+ *	}
+ *    };
+ *
+*/
+/* next is for each fontfamily the ascent value as given by the 
+* medium sized bold x-font (the regular font has the same
+* ascent value for both the medium and the large size Century font).
+* it is use in the check for the fontsize (small, medium, large)
+*/
+/*    static int medium_fontascent[4] = {
+ *	14, 14, 16, 15
+ *    };
+ *
+*/
+/* for each fontfamily, for each fontsize, and for each font style
+ * give the ascent value, so the output from Postscript is correct.
+ * If the value is given between parenthesis, then it is different
+ * from the value as stored in the x-font.
+ * Note that this is a fix, and need to be changed, if the browser
+ * is fixed (in the current version 1.2 the baseline of various fonts
+ * is not aligned very well).
+ */
+/*    static int fontascent[4][3][17] = {
+ * rg, itl, bld, fix,  h1,  h2,  h3,  h4,  h5,  h6,
+ *	  add, pla, lis, fixbold, fixital, plabold, plaital, */
+/* times */
+/*	{
+ *	    {12, 11, 12, 10, 15, 14, 12, 10, 8, 6, 11, 9, 10, 10, 10, 9, 9},
+ *	    {13, 13, 14, 12, 20, 15, 14, 12, 10, 8, 13, 10, 10, 12, 12, 10, 10},
+ *	    {16, 15, 15, 13, 21, 20, 15, 15, 14, 12, 15, 13, 10, 13, 13, 13, 13}
+ *	},
+*/
+/* helvetica */
+/*	{
+ *	    {12, 12, 12, 10, 15, 14, 12, 10, 9, 7, 12, 9, 10, 10, 10, 9, 9},
+ *	    {14, 14, 14, 12, 22, 15, 14, 12, 10, 9, 14, 10, 10, 12, 12, 10, 10},
+ *	    {16, 16, 16, 13, 22, 22, 16, 15, 14, 12, 16, 13, 10, 13, 13, 13, 13}
+ *	},
+*/
+/* new century schoolbook */
+/*	{
+ *	    {12, 12, 13, 10, 16, 14, 13, 10, 9, 7, 12, 9, 10, 10, 10, 9, 9},
+ *	    {16, 14, 16, 13, 22, 16, 14, 13, 10, 9, 14, 10, 10, 13, 13, 10, 10},
+ *	    {17, 16, 17, 13, 22, 22, 17, 16, 14, 13, 16, 13, 10, 13, 13, 13, 13}
+ *	},
+*/
+/* lucida bright */
+/*	{
+ *	    {11, 11, 11, 11, 15, 14, 11, 10, 9, 7, 11, 9, 10, 11, 10, 9, 9},
+ *	    {14, 15, 14, 13, 20, 15, 14, 11, 10, 7, 15, 11, 10, 13, 13, 11, 10},
+ *	    {17, 17, 17, 16, 21, 20, 17, 15, 14, 11, 17, 14, 10, 16, 13, 14, 13}
+ *	}
+ *    };
+ *
+*/
+/* NULL case - reflush old font or the builtin default: */
+/*    if ((hw == NULL) || (font == NULL)) {
+ *	if (PS_oldfs != 0)
+ *	    PSprintf( "%2s %d SF\n", fnchar[PS_oldfn], PS_oldfs);
+ *	return;
+ *    }
+*/
+/* added the next line in case xmosaic version 199.4 has more fonts */
+/*    style = 3;
+ *    if (font == hw->html.font) {
+ *	style = 0;
+ *    } else if (font == hw->html.italic_font) {
+ *	style = 1;
+ *    } else if (font == hw->html.bold_font) {
+ *	style = 2;
+ *    } else if (font == hw->html.fixed_font) {
+ *	style = 3;
+ *    } else if (font == hw->html.header1_font) {
+ *	style = 4;
+ *    } else if (font == hw->html.header2_font) {
+ *	style = 5;
+ *    } else if (font == hw->html.header3_font) {
+ *	style = 6;
+ *    } else if (font == hw->html.header4_font) {
+ *	style = 7;
+ *    } else if (font == hw->html.header5_font) {
+ *	style = 8;
+ *    } else if (font == hw->html.header6_font) {
+ *	style = 9;
+ *    } else if (font == hw->html.address_font) {
+ *	style = 10;
+ *    } else if (font == hw->html.plain_font) {
+ *	style = 11;
+ *    } else if (font == hw->html.listing_font) {
+ *	style = 12;
+ *    } else if (font == hw->html.fixedbold_font) {
+ *	style = 13;
+ *    } else if (font == hw->html.fixeditalic_font) {
+ *	style = 14;
+ *    } else if (font == hw->html.plainbold_font) {
+ *	style = 15;
+ *    } else if (font == hw->html.plainitalic_font) {
+ *	style = 16;
+ *    }
+ *
+*/
+/* check size, by looking at the size of the regular font */
+/*    size = 1;
+ *    if (hw->html.bold_font->ascent > medium_fontascent[fontfamily]) {
+*/
+/* large font */
+/*	size = 2;
+ *    } else if (hw->html.bold_font->ascent < medium_fontascent[fontfamily]) {
+*/
+/* small font */
+/*	size = 0;
+ *    }
+ *    fn = fontstyle[style];
+ *    fs = fontsizes[fontfamily][size][style];
+ *    PS_fontascent = fontascent[fontfamily][size][style];
+ *
+ *    if (fn != PS_oldfn || fs != PS_oldfs) {
+ *	PSprintf( "%2s %d SF\n", fnchar[fn], fs);
+ *	PS_oldfn=fn, PS_oldfs=fs;
+ *    }
+*/
 }
 
 /* PSshowpage - end of page function
@@ -506,8 +508,7 @@ static void PSnewpage(void)
 
     PSprintf("%%%%Page: %d %d\n", PS_curr_page, PS_curr_page);
     PSprintf("save\n");
-    if (HTML_Print_Headers)
-	PSprintf("%d ", PS_curr_page);
+    PSprintf("%d ", PS_curr_page);
     PSprintf("NP\n");
     PSfont( NULL, NULL, 0);	/* force re-flush of last font used */
 }
@@ -695,7 +696,6 @@ static void PSheader(char *title, int font, char *url, char *time_str)
     /* Output the newpage definition. */
     
     PSprintf("/NP {");
-    if (HTML_Print_Headers) {
 	PSprintf("gsave 0.4 setlinewidth\n");
 	PSprintf("  newpath %.2f %.2f M %.2f 0 RL stroke", 
 		 page_dimens.left_margin,
@@ -716,7 +716,6 @@ static void PSheader(char *title, int font, char *url, char *time_str)
 	PSprintf("  (%s) dup stringwidth pop %.2f E sub %.2f M S grestore\n",
 		 time_str, page_dimens.left_margin + page_dimens.text_width,
 		 page_dimens.bot_margin - 12);
-    }
     PSprintf("  %.2f %.2f translate %.5f %.5f scale } D\n",
 	     page_dimens.left_margin, 
 	     page_dimens.bot_margin + page_dimens.text_height,
@@ -786,7 +785,7 @@ static int has_footnote(struct ele_rec *el)
     if (el == NULL)
 	return 0;
 
-    anchorHRef = el->anchor_tag_ptr->anchor_href;
+    anchorHRef = el->anchor_tag_ptr->anc_href;
     if (anchorHRef != NULL) {
 	switch (el->type) {
 	case E_TEXT:
@@ -795,10 +794,10 @@ static int has_footnote(struct ele_rec *el)
 		if (next == NULL) {
 		    rc = 1;
 		    break;
-		} else if (next->anchor_tag_ptr->anchor_href == NULL) {
+		} else if (next->anchor_tag_ptr->anc_href == NULL) {
 			rc = 1;
 			break;
-		} else if (!STREQ(next->anchor_tag_ptr->anchor_href, anchorHRef)){
+		} else if (!STREQ(next->anchor_tag_ptr->anc_href, anchorHRef)){
 		    rc = 1;
 		    break;
 		} else if (   (next->type == E_TEXT) || (next->type == E_IMAGE)) {
@@ -834,10 +833,7 @@ static void PSfootnote(char *href, double height)
 		footnotes = (char **)realloc((void *)footnotes,
 				     (ftn_array_size * sizeof(char *)));
 	}
-	if (footnotes == NULL) {
-		fprintf(stderr, "PSfootnote realloc failed\n");
-		return;
-	}
+	CHECK_OUT_OF_MEM(footnotes);
     }
     footnotes[n_saved_ftns++] = href;
 }
@@ -866,10 +862,7 @@ static void PStext(HTMLWidget hw, struct ele_rec *eptr, int fontfamily, String s
        every character stored as an octal escape (worst case scenario). */
 
     s2 = (String) malloc(strlen(s) * 4 + 1);
-    if (s2 == NULL) {
-	fprintf(stderr, "PStext malloc failed\n");
-	return;
-    }
+	CHECK_OUT_OF_MEM(s2);
 
     /*  For each char in s, if it is a special char, insert "\"
      *  into the new string s2, then insert the actual char
@@ -890,8 +883,8 @@ static void PStext(HTMLWidget hw, struct ele_rec *eptr, int fontfamily, String s
     }
     *(stmp) = '\0';
     PSprintf("(%s)%c\n", s2, (underline)?'U':'S');
-    if (HTML_Print_Footers && has_footnote(eptr)) 
-	PSfootnote(eptr->anchor_tag_ptr->anchor_href, 0.7 * ascent);
+    if (has_footnote(eptr)) 
+	PSfootnote(eptr->anchor_tag_ptr->anc_href, 0.7 * ascent);
     free(s2);
 }
 
@@ -947,7 +940,7 @@ static void PShrule(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
  */
 static void PStable(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 {
-    struct table_rec	*tptr = eptr->table_data;
+    struct _TableRec	*tptr = eptr->table_data;
     int width  = tptr->width;
     int height = tptr->height;
 
@@ -1156,7 +1149,7 @@ static void PScolor_image(void)
  * spits out code for the colormap of the following image
  * if !color, it spits out a mono-ized graymap
 */
-static void PScolormap(int color, int nc, int *rmap, int *gmap, int *bmap) 
+static void PScolormap(int color, int nc, XColor *cmap) 
 {
     int i;
 
@@ -1168,10 +1161,10 @@ static void PScolormap(int color, int nc, int *rmap, int *gmap, int *bmap)
 
     for (i=0; i<nc; i++) {
 	if (color)  
-	    PSprintf("%02x%02x%02x ", rmap[i]>>8,
-		     gmap[i]>>8, bmap[i]>>8);
+	    PSprintf("%02x%02x%02x ", cmap[i].red>>8,
+		     cmap[i].green>>8, cmap[i].blue>>8);
 	else  
-	    PSprintf("%02x ", MONO(rmap[i], gmap[i], bmap[i]));
+	    PSprintf("%02x ", MONO(cmap[i].red, cmap[i].green, cmap[i].blue));
 	if ((i%10) == 9)
 	    PSprintf("\n");
     }
@@ -1291,7 +1284,7 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 {
     ImageInfo 		*img = eptr->pic_data;
     unsigned char 	*imgp = img->image_data;
-    int 		anchor = (eptr->anchor_tag_ptr->anchor_href != NULL); 	
+    int 		anchor = (eptr->anchor_tag_ptr->anc_href != NULL); 	
     int 		ncolors = img->num_colors;
     int 		i, j;
     int 		w = img->width;
@@ -1302,13 +1295,13 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 
 
     /* Isgray returns true if the nth color index is a gray value */
-#define Isgray(i,n) (i->reds[n]==i->greens[n] && i->reds[n]==i->blues[n])
+#define Isgray(i,n) (i->colrs[n].red==i->colrs[n].green && i->colrs[n].red==i->colrs[n].blue)
     /* Is_bg returns true if the nth color index is the screen background */
-#define Is_bg(i,n)  (i->reds[n]==bg_color.red &&			\
-		i->greens[n]==bg_color.green && i->blues[n]==bg_color.blue)
+#define Is_bg(i,n)  (i->colrs[n].red==bg_color.red &&			\
+		i->colrs[n].green==bg_color.green && i->colrs[n].blue==bg_color.blue)
     /* Is_fg returns true if the nth color index is the screen foreground */
-#define Is_fg(i,n)  (i->reds[n]==fg_color.red &&			\
-		i->greens[n]==fg_color.green && i->blues[n]==fg_color.blue)
+#define Is_fg(i,n)  (i->colrs[n].red==fg_color.red &&			\
+		i->colrs[n].green==fg_color.green && i->colrs[n].blue==fg_color.blue)
 
 
     PSmove_offset(eptr->baseline);
@@ -1394,10 +1387,10 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 	
 	/*  set if color#0 is 'white' */
 	if ((ncolors == 2 &&
-	     MONO(img->reds[0], img->greens[0],img->blues[0]) >
-	     MONO(img->reds[1], img->greens[1], img->blues[1])) ||
+	     MONO(img->colrs[0].red, img->colrs[0].green,img->colrs[0].blue) >
+	     MONO(img->colrs[1].red, img->colrs[1].green, img->colrs[1].blue)) ||
 	    (ncolors == 1 && 
-	     MONO(img->reds[0], img->greens[0],img->blues[0]) >
+	     MONO(img->colrs[0].red, img->colrs[0].green,img->colrs[0].blue) >
 	     MONO(127, 127, 127) ))
 	{
 	    flipbw=1; 
@@ -1423,7 +1416,7 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 	if (colorps)
 	    PScolor_image();
 	
-	PScolormap(colorps, ncolors, img->reds, img->greens, img->blues);
+	PScolormap(colorps, ncolors, img->colrs);
 	PSrle_cmapimage(colorps);
 
 	/*  dimensions of data */
@@ -1433,10 +1426,7 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 	PSprintf("rlecmapimage\n");
 	
 	rleline = (unsigned char *) malloc(w * 2);
-	if (!rleline) {
-		fprintf(stderr,"failed to malloc space for rleline\n");
-		return;
-	}
+	CHECK_OUT_OF_MEM(rleline);
 
 	for (i=0; i<h && err != EOF; i++) {
 	    rlen = PSrle_encode(imgp, rleline, w);
@@ -1454,9 +1444,9 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
 	
     /*  move currentpoint just right of image */
     PSprintf("%d 0 R\n", w + extra); 
-    if (HTML_Print_Footers && has_footnote(eptr)) {
+    if (has_footnote(eptr)) {
 	PSmove_offset(0);
-	PSfootnote(eptr->anchor_tag_ptr->anchor_href, 2.0);
+	PSfootnote(eptr->anchor_tag_ptr->anc_href, 2.0);
     }
 
     /* forget about the macro's */
@@ -1489,12 +1479,10 @@ static void PSimage(HTMLWidget hw, struct ele_rec *eptr, int fontfamily)
  *	3: lucida
  */
 String ParseTextToPSString(HTMLWidget	 	hw, 
-			   struct ele_rec	*elist,
 			   struct ele_rec	*startp,
 			   struct ele_rec	*endp,
 			   int			start_pos, 
 			   int			end_pos,
-			   int			space_width,
 			   int			lmargin,
 			   int			fontfamily,
 			   char   		*url,
@@ -1503,7 +1491,6 @@ String ParseTextToPSString(HTMLWidget	 	hw,
     int			xpos, ypos, epos;
     int			height;
     double		pagewidth;
-    int			line = -1;
     struct		ele_rec	*eptr;
     struct		ele_rec	*start;
     struct		ele_rec	*end;
@@ -1526,18 +1513,8 @@ String ParseTextToPSString(HTMLWidget	 	hw,
 		   NULL);
     fg_color.pixel = fg_pixel;
     bg_color.pixel = bg_pixel;
-    XQueryColor(XtDisplay(hw->html.view),
-		(installed_colormap ?
-		 installed_cmap :
-		 DefaultColormap(XtDisplay(hw->html.view),
-				DefaultScreen(XtDisplay(hw->html.view)))),
-		&fg_color);
-    XQueryColor(XtDisplay(hw->html.view),
-		(installed_colormap ?
-		 installed_cmap :
-		 DefaultColormap(XtDisplay(hw->html.view),
-				 DefaultScreen(XtDisplay(hw->html.view)))),
-		&bg_color);
+    XQueryColor(XtDisplay(hw->html.view), hw->core.colormap , &fg_color);
+    XQueryColor(XtDisplay(hw->html.view), hw->core.colormap , &bg_color);
     
     /*  this piece of code is needed if the user selects a portion
      *  of the document with the mouse.
@@ -1590,7 +1567,7 @@ String ParseTextToPSString(HTMLWidget	 	hw,
     Pixels_Page = (int) (page_dimens.text_height / Points_Pixel);		
 	
     PSinit();
-    PSheader(hw->html.title, fontfamily, url, time_str);
+    PSheader("title:" /*hw->html.title*/, fontfamily, url, time_str);
     PSnewpage();
 
     last = start;
@@ -1598,17 +1575,16 @@ String ParseTextToPSString(HTMLWidget	 	hw,
 
     while ((eptr != NULL) && (eptr != end)) {
 	/* check if this is a newline */
-	if (line != eptr->line_number) {
+/* ################## */
 	    /* calculate max height */
 	    height = 0;
 	    footnotes_this_line = 0;
-	    line = eptr->line_number;
 	    tmpptr = eptr;
-	    while (tmpptr != NULL && tmpptr->line_number == line) {
+	    while (tmpptr != NULL && tmpptr->type != E_LINEFEED) {
 		if (tmpptr->height > height)
 		    height = tmpptr->height;
 		tmpptr = tmpptr->next;
-		if (HTML_Print_Footers && has_footnote(tmpptr))
+		if (has_footnote(tmpptr))
 		    footnotes_this_line++;
 	    }
 	    ypos = eptr->y - PS_page_offset ;
@@ -1634,8 +1610,8 @@ String ParseTextToPSString(HTMLWidget	 	hw,
 	    }
 	    footnotes_this_page += footnotes_this_line;
 	    PSmoveto( xpos, ypos);
-	}
 
+/* ######### */
 	switch (eptr->type) {
 	case E_TEXT:
 	    PStext(hw, eptr, fontfamily,
@@ -1658,6 +1634,12 @@ String ParseTextToPSString(HTMLWidget	 	hw,
 	case E_WIDGET:
 	    PSwidget(hw, eptr, fontfamily);
 	    break;
+	case E_CR:
+	case E_ANCHOR:
+	case E_CELL_TABLE:
+	case E_OBJECT:
+	case E_APPLET:
+		break;
 	}
 	last = eptr;
 	eptr = eptr->next;
