@@ -1,25 +1,22 @@
 /* Most of code is rewrite from scratch. Lot of code come from NCSA Mosaic */
 
-#include "copyright.ncsa"
+#include "../Copyrights/copyright.ncsa"
 
-/* HTMLtext.c
- * Author: Gilles Dauphin
- * Version 3.0 [Sep96]
- *
- * Copyright (C) 1996 - G.Dauphin
+/* Copyright (C) 1996 - G.Dauphin
  * See the file "license.mMosaic" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES. 
- *
- * Bug report :  dauphin@sig.enst.fr
- */
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+*/
 
-#include "../libmc/mc_defs.h"
-#include "HTMLP.h"
-#include "HTMLPutil.h"
 #include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include "HTMLamp.h"
+
+#include "HTMLmiscdefs.h"
+#include "HTMLparse.h"
+#include "HTMLP.h"
+#include "HTMLPutil.h"
+
+#define HTAB_WIDTH 8
 
 #define SKIPWHITE(s)    while( (((unsigned char)*s) < 128) && (isspace(*s)) ) s++
 #define SKIPNONWHITE(s) while( (((unsigned char)*s) > 127) ||  \
@@ -47,26 +44,33 @@ void LinefeedPlace(HTMLWidget hw, struct mark_up * mptr,
 	/* We need to center or right adjust the element here. */
 	/* compute the total with and the remaining x offset to center. */
 
-	if ((pcc->div == DIV_ALIGN_CENTER) || (pcc->div == DIV_ALIGN_RIGHT)) {
+	if ((pcc->div == HALIGN_CENTER) || (pcc->div == HALIGN_RIGHT)) {
 		adjx = pcc->cur_line_width - 
 			(pcc->x - pcc->eoffsetx - pcc->left_margin) ;
-		if(pcc->div == DIV_ALIGN_CENTER)
+		if(pcc->div == HALIGN_CENTER)
 			adjx = adjx/2;
 	}
 
 	if (pcc->cw_only) {	/* compute width only , dont create Element*/
 		if (pcc->x > pcc->computed_max_x)
 			pcc->computed_max_x = pcc->x;
-		pcc->y = pcc->y + pcc->cur_line_height;
-		pcc->have_space_after = 0;
-		pcc->x = pcc->left_margin + pcc->eoffsetx;
+		pcc->y = pcc->y + pcc->cur_line_height;	 /* LF equiv. */
+		pcc->x = pcc->left_margin + pcc->eoffsetx +
+			pcc->float_align_left;		 /* CR equiv. */
+/* look at float object */
+		UnsetFloatAlignLeft(pcc); /* test pcc->fasl->fa->trap_boty > pcc->y */
+					/* alors on change pcc->float_align_left */
+
+		UnsetFloatAlignRight(pcc); /* test pcc->fasr->fa->trap_boty > pcc->y */
 		pcc->cur_baseline = pcc->cur_font->ascent;
-		pcc->cur_line_height=pcc->cur_font->ascent+pcc->cur_font->descent;
+/* ###		pcc->cur_line_height=0;		/* #### */
+		pcc->cur_line_height=pcc->cur_font->ascent+pcc->cur_font->descent; /* ##### */
+		pcc->have_space_after = 0;
 		pcc->is_bol = 1;
 		return;
 #ifdef REMEMBER
 if (pcc->cw_only) {	/* compute width only , dont create Element*/
-/* deb_pcc.computed_min_x = 0; /* celui-ci pour MOT IMAGE APROG */
+/* deb_pcc.computed_min_x = 0; */ /* celui-ci pour MOT IMAGE OBJECT APPLET*/
 /* attention aussi au increment genre MENU*/
 /* UL OL etc... */
 }
@@ -76,14 +80,14 @@ if (pcc->cw_only) {	/* compute width only , dont create Element*/
 	/* linefeed is at end of line */
 	eptr = CreateElement(hw, E_LINEFEED, pcc->cur_font,
 		pcc->x, pcc->y, 0, pcc->cur_line_height, pcc->cur_baseline, pcc);
-/*	eptr->anchor_tag_ptr = pcc->anchor_tag_ptr; */
-		
-	if(adjx > 0 ){
-		/* adjust */
+
+	if(adjx > 0 ){ /* adjust */
 		/* Back to the list until CR and adjust each x with the adjx. */
 		septr = eptr;
 		while( septr && (septr->type != E_CR) ){
 			septr->x = septr->x + adjx;
+			if(septr->widget_data) /* Also move the widget */
+				septr->widget_data->x = septr->x;
 			septr = septr->prev;
 		}
 		pcc->x = pcc->x + adjx;
@@ -93,19 +97,21 @@ if (pcc->cw_only) {	/* compute width only , dont create Element*/
 	if( pcc->x  > pcc->max_width_return )
 		pcc->max_width_return = pcc->x ;
 
-	(pcc->line_number)++;
-	pcc->y = pcc->y + pcc->cur_line_height;
+	pcc->y = pcc->y + pcc->cur_line_height;		/* LF equiv. */
+	pcc->x = pcc->left_margin + pcc->eoffsetx +
+			pcc->float_align_left;		 /* CR equiv. */
+	UnsetFloatAlignLeft(pcc); /* test pcc->fasl->fa->trap_boty > pcc->y */
+	UnsetFloatAlignRight(pcc);/* test pcc->fasr->fa->trap_boty > pcc->y */
 	pcc->have_space_after = 0;
-	pcc->x = pcc->left_margin + pcc->eoffsetx;
 	pcc->cur_baseline = pcc->cur_font->ascent;
-	pcc->cur_line_height = pcc->cur_font->ascent+pcc->cur_font->descent;
+/*###	pcc->cur_line_height=0;		/* #### */
+	pcc->cur_line_height=pcc->cur_font->ascent+pcc->cur_font->descent;
 	pcc->is_bol = 1;
 	/* CR is at begin of line */
 	eptr = CreateElement(hw, E_CR, pcc->cur_font,
 		pcc->x, pcc->y,
 		0, pcc->cur_line_height,
 		pcc->cur_baseline, pcc);
-/*	eptr->anchor_tag_ptr = pcc->anchor_tag_ptr; */
 }
 
 /* Redraw a linefeed. Basically a filled rectangle at the end of a line. */
@@ -127,7 +133,6 @@ static void Translate_Nbsp( char * t)
 	}
 }
 		
-
 /* remove blank, LF, CR, TAB */
 static char ** split_in_word( char * text, unsigned int * nw,
 		int * hsb4, int * hsa)
@@ -187,9 +192,7 @@ void Set_E_TEXT_Element(HTMLWidget hw,
 
 	len = strlen(text) + 1;
 	eptr->edata = strdup(text);
-	if (eptr->edata == NULL) {
-		MEM_OVERFLOW;
-	}
+	CHECK_OUT_OF_MEM(eptr->edata);
 	eptr->edata_len = len;
 			/* if this is an anchor, puts its href and name */
 			/* values into the element.  */
@@ -251,9 +254,15 @@ void PartOfTextPlace(HTMLWidget hw, 	/* the widget */
 
 	words = split_in_word(text,&nword, &have_space_b4, &have_space_after);
 	if (nword == 0 ){
+		pcc->have_space_after = have_space_after;
 		free(text);
 		return;
 	}
+	if (pcc->have_space_after && !have_space_b4)
+		have_space_b4 = 1;
+	if (is_bol)		/* si on est en begin of line */
+		have_space_b4 = 0;
+
 				/* alloc enought space to compose a line */
 	composed_line = (char*) malloc( strlen(text) + nword*3 + 1);
 	composed_line[0]='\0';
@@ -261,10 +270,6 @@ void PartOfTextPlace(HTMLWidget hw, 	/* the widget */
 	the_word = (char*) malloc( strlen(text) + nword*3 + 1);
 	the_word[0]='\0';
 
-	if (pcc->have_space_after && !have_space_b4)
-		have_space_b4 = 1;
-	if (is_bol)		/* si on est en begin of line */
-		have_space_b4 = 0;
 	for(i = 0; i < nword; i++){	/* remplir la ligne de mots */
 		if(have_space_b4){
 			the_word[0]= ' ';
@@ -405,14 +410,13 @@ void PartOfPreTextPlace(HTMLWidget hw, 	/* the widget */
 	char *end;
 	struct ele_rec *eptr;
 	int ntab, char_cnt;
-	int dir, ascent, descent;
-        XCharStruct all;
 	char *line;
 	int tmp_cnt=0;
 	int cur_char_in_line=0;
 	int line_str_len;
 	int i;
 	int font_height;
+	int line_width=0;
 
 	font_height = pcc->cur_font->ascent+pcc->cur_font->descent;
 	line = (char*) malloc(COMP_LINE_BUF_LEN);
@@ -427,12 +431,17 @@ void PartOfPreTextPlace(HTMLWidget hw, 	/* the widget */
 		} 
 		if(*end == '\n'){	/*line break */
 			if (line[0] != '\0') {
-				XTextExtents(pcc->cur_font, line, strlen(line),
-					&dir, &ascent, &descent, &all);
+				line_width = XTextWidth(pcc->cur_font,
+					line, strlen(line));
+			    if(pcc->computed_min_x < 
+		              (line_width+pcc->eoffsetx+pcc->left_margin)){
+			         pcc->computed_min_x= line_width + pcc->eoffsetx +
+					      pcc->left_margin;
+			    }
 			    if(!pcc->cw_only){
 				eptr = CreateElement(hw, E_TEXT,
 					pcc->cur_font,
-                               		pcc->x, pcc->y, all.width,
+                               		pcc->x, pcc->y, line_width,
 					pcc->cur_font->ascent+pcc->cur_font->descent,
 					pcc->cur_font->ascent,pcc);
 				Set_E_TEXT_Element(hw, eptr, line,pcc);
@@ -444,7 +453,7 @@ void PartOfPreTextPlace(HTMLWidget hw, 	/* the widget */
 				pcc->pf_lf_state = 0;
 			}
 			end++;
-			pcc->x = pcc->x + all.width;
+			pcc->x = pcc->x + line_width;
 			LinefeedPlace(hw, mptr,pcc); /* line feed change le contexte*/
 			char_cnt =0;
 			line[0] = '\0';
@@ -457,7 +466,7 @@ void PartOfPreTextPlace(HTMLWidget hw, 	/* the widget */
 		 * Break on linefeeds, they must be done separately
 		 */
 		if (*end == '\t') {
-			tmp_cnt= ((char_cnt/8) + 1) * 8;
+			tmp_cnt= ((char_cnt/HTAB_WIDTH) + 1) * HTAB_WIDTH;
 			ntab = tmp_cnt - char_cnt;
 			char_cnt += ntab;
 			if (char_cnt +1 > line_str_len) {
@@ -483,12 +492,12 @@ void PartOfPreTextPlace(HTMLWidget hw, 	/* the widget */
 		end++;
 	}
 	if (line[0] != '\0') {
-		XTextExtents(pcc->cur_font, line, strlen(line),
-			&dir, &ascent, &descent, &all);
+		line_width = XTextWidth(pcc->cur_font,
+				line, strlen(line));
 	    if(!pcc->cw_only){
 		eptr = CreateElement(hw, E_TEXT,
 			pcc->cur_font,
-               		pcc->x, pcc->y, all.width,
+               		pcc->x, pcc->y, line_width,
 			pcc->cur_font->ascent+pcc->cur_font->descent,
 			pcc->cur_font->ascent,pcc);
 		Set_E_TEXT_Element(hw, eptr, line,pcc);
@@ -497,10 +506,12 @@ void PartOfPreTextPlace(HTMLWidget hw, 	/* the widget */
                      if (pcc->cur_line_height < font_height)
                                 pcc->cur_line_height = font_height;
             }
-		pcc->x = pcc->x + all.width;
+		pcc->x = pcc->x + line_width;
 		pcc->is_bol =0;
 		pcc->pf_lf_state = 0;
 	}
+	if (pcc->x > pcc->computed_max_x)
+		pcc->computed_max_x = pcc->x;
 	pcc->have_space_after = 0;
 	free(line);
 }
@@ -546,6 +557,9 @@ void PartialRefresh(HTMLWidget hw, struct ele_rec *eptr,
 	x = x - hw->html.scroll_x;
 	y = y - hw->html.scroll_y;
 
+	if (y > hw->html.view_height || y + eptr->height < 0)
+		return;                 /* not visible */
+
 	/*
 	 * May be safe to used the cached full width of this
 	 * string, and thus avoid a call to XTextExtents
@@ -560,15 +574,14 @@ void PartialRefresh(HTMLWidget hw, struct ele_rec *eptr,
 
 	height = eptr->height;
 	if (height > 0) {
-		if(!hw->html.bg_image)
+		if(!hw->html.cur_res.have_bgima)
 			XFillRectangle(XtDisplay(hw), XtWindow(hw->html.view),
 				hw->html.drawGC, x, y,
 				(unsigned int)all.width, (unsigned int)height);
 	}
 	width = all.width;
 
-	if(bg!=hw->html.view->core.background_pixel ||
-	   NoBodyImages((Widget)hw) || !hw->html.bg_image) {
+	if(bg!=hw->html.view->core.background_pixel || !hw->html.cur_res.have_bgima) {
 		XSetForeground(XtDisplay(hw), hw->html.drawGC, bg);
 		XSetBackground(XtDisplay(hw), hw->html.drawGC, fg);
 
@@ -583,22 +596,16 @@ void PartialRefresh(HTMLWidget hw, struct ele_rec *eptr,
 			    x, y + eptr->baseline,
 			    (char *)tdata, tlen);
 	} else {
-		XSetForeground(XtDisplay(hw), hw->html.drawGC, bg);
-		XSetBackground(XtDisplay(hw), hw->html.drawGC, fg);
-		XFillRectangle(XtDisplay(hw), XtWindow(hw->html.view),
-			       hw->html.drawGC,
-			       x, y,
-			       width, height);
+/* XFillRectangle(XtDisplay(hw), XtWindow(hw->html.view), hw->html.drawGC,
+	       x, y, width, height);
+*/
+		HTMLDrawBackgroundImage(hw, (x<0 ? 0 : x), (y<0 ? 0 : y),
+			(x<0 ? (width+x) : width), height );
 		XSetForeground(XtDisplay(hw), hw->html.drawGC, fg);
 		XSetBackground(XtDisplay(hw), hw->html.drawGC, bg);
-		HTMLDrawBackgroundImage((Widget)hw,
-					(x<0 ? 0 : x), (y<0 ? 0 : y),
-					(x<0 ? (width+x) : width),
-					height );
 		XDrawString(XtDisplay(hw), XtWindow(hw->html.view),
-			    hw->html.drawGC,
-			    x, y + eptr->baseline,
-			    (char *)tdata, tlen);
+			hw->html.drawGC, x, y + eptr->baseline,
+			(char *)tdata, tlen);
 	}
 	if (eptr->underline_number) {
 		int i, ly;
@@ -713,7 +720,7 @@ void HRuleRefresh( HTMLWidget hw, struct ele_rec *eptr)
 
 	/* blank out area */
 	XSetForeground(XtDisplay(hw), hw->html.drawGC, eptr->bg);
-	if(!hw->html.bg_image)
+	if(!hw->html.cur_res.have_bgima)
 		XFillRectangle(XtDisplay(hw), XtWindow(hw->html.view),
 			hw->html.drawGC, x1, y1, width, height);
 	y1 = y1 + (height / 2) - 1;
@@ -751,7 +758,7 @@ void BulletPlace( HTMLWidget hw, struct mark_up * mptr,
 	}
 /*	pcc->x = pcc->x + pcc->cur_font->max_bounds.width; ###*/
 	pcc->x = pcc->x ;
-	pcc->is_bol = 0;
+	pcc->is_bol = 1;
 }
 
 /* Redraw a formatted bullet element */
