@@ -1,5 +1,5 @@
 /* Please read copyright.tmpl. Don't remove next line */
-#include "copyright.ncsa"
+#include "../Copyrights/copyright.ncsa"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -7,7 +7,9 @@
 #include <stdlib.h>
 
 #include <Xm/XmAll.h>
-#include "../libmc/mc_defs.h"
+
+#include "HTMLmiscdefs.h"
+#include "HTMLparse.h"
 #include "HTMLP.h"
 #include "HTMLPutil.h"
 
@@ -28,36 +30,10 @@
 
 extern char *ParseMarkTag();
 
-static Boolean ModifyIgnore = False;
-
 char ** 	ParseCommaList( char *str, int *count);
 void 		FreeCommaList( char **list, int cnt);
 char * 		MapOptionReturn( char *val, char **mapping);
 
-static char traversal_table[] = "\
-~Shift ~Meta ~Ctrl <Key> Tab:  traversal_forward()\n\
-Shift ~Meta ~Ctrl <Key> Tab:   traversal_back()\n\
-Ctrl <Key> Tab:    traversal_end()\
-";
-
-static char text_translations[] = "\
-           ~Meta ~Alt Ctrl<Key>u:       beginning-of-line()             \
-                                        delete-to-end-of-line()         \n\
-           ~Meta ~Alt Ctrl<Key>k:       delete-to-end-of-line()         \n\
-           ~Meta ~Alt Ctrl<Key>a:       beginning-of-line()             \n\
-           ~Meta ~Alt Ctrl<Key>e:       end-of-line()                   \n\
-           ~Meta ~Alt Ctrl<Key>w:       key-select()                    \
-                                        delete-selection()              \n\
-           ~Meta ~Alt Ctrl<Key>y:       paste-clipboard()               \n\
-         Meta ~Ctrl       <Key>d:       delete-next-word()              \n\
-          Alt ~Ctrl       <Key>d:       delete-next-word()              \n\
-           ~Meta ~Alt Ctrl<Key>d:       delete-next-character()         \n\
-     Meta ~Ctrl<Key>osfBackSpace:       delete-previous-word()          \n\
-      Alt ~Ctrl<Key>osfBackSpace:       delete-previous-word()          \n\
-        Meta ~Ctrl<Key>osfDelete:       delete-next-word()              \n\
-         Alt ~Ctrl<Key>osfDelete:       delete-next-word()              \n\
-                      <Btn1Down>:       take_focus() grab-focus() traversal_current()";
-
 
 void AddNewForm( HTMLWidget hw, FormInfo *fptr)
 {
@@ -279,6 +255,31 @@ int CollectSubmitInfo( FormInfo *fptr, char ***name_list, char ***value_list)
 	return(cbdata.attribute_count);
 }
 
+void InputImageSubmitForm( FormInfo *fptr, XEvent *event, HTMLWidget hw)
+{
+	WbFormCallbackData cbdata;
+	int i, cnt;
+	char **name_list;
+	char **value_list;
+	char valstr[100];
+
+	cbdata.event = event;
+	cbdata.href = fptr->action;
+        cbdata.method = fptr->method;
+        cbdata.enctype = fptr->enctype;
+
+	name_list = NULL;
+	value_list = NULL;
+	fptr->button_pressed = NULL;
+
+	cbdata.attribute_count = CollectSubmitInfo(fptr, &name_list, &value_list);
+	cbdata.attribute_names = name_list;
+	cbdata.attribute_values = value_list;
+
+	XtCallCallbackList ((Widget)hw, hw->html.form_callback,
+		(XtPointer)&cbdata);
+}
+
 void ImageSubmitForm( FormInfo *fptr, XEvent *event, int x, int y)
 {
 	HTMLWidget hw = (HTMLWidget)(fptr->hw);
@@ -290,10 +291,8 @@ void ImageSubmitForm( FormInfo *fptr, XEvent *event, int x, int y)
 
 	cbdata.event = event;
 	cbdata.href = fptr->action;
-	cbdata.format = fptr->format;
         cbdata.method = fptr->method;
         cbdata.enctype = fptr->enctype;
-        cbdata.enc_entity = fptr->enc_entity;
 
 	name_list = NULL;
 	value_list = NULL;
@@ -341,9 +340,7 @@ void CBSubmitForm( Widget w, caddr_t client_data, caddr_t call_data)
 	cbdata.event = pb->event;
 	cbdata.href = fptr->action;
         cbdata.method = fptr->method;
-	cbdata.format = fptr->format;
         cbdata.enctype = fptr->enctype;
-        cbdata.enc_entity = fptr->enc_entity;
 	fptr->button_pressed = w;
 
 	cbdata.attribute_count = CollectSubmitInfo(fptr,
@@ -468,10 +465,6 @@ void CBPasswordModify( Widget w, caddr_t client_data, caddr_t call_data)
 	tv->doit = False;
 	 */
 
-	/* Ignore when ModifyIgnore is true */
-	if (ModifyIgnore == True)
-		return;
-
 	/* only accept text modification of password fields */
 	if (tv->reason != XmCR_MODIFYING_TEXT_VALUE)
 		return;
@@ -598,13 +591,12 @@ void CBPasswordModify( Widget w, caddr_t client_data, caddr_t call_data)
  * RETURN was hit in a textfield in a form.
  * If this is the only textfield in this form, submit the form.
  */
-void CBActivateField( Widget w, caddr_t client_data, caddr_t call_data)
+void CBActivateField( Widget w, caddr_t clid, caddr_t calld)
 {
-	FormInfo *fptr = (FormInfo *)client_data;
+	FormInfo *fptr = (FormInfo *)clid;
 	HTMLWidget hw = (HTMLWidget)(fptr->hw);
 	WidgetInfo *wptr;
 	int cnt, count;
-	XmAnyCallbackStruct *cb = (XmAnyCallbackStruct *)call_data;
 
 	/*
 	 * Terminate the form if it was never properly terminated.
@@ -651,17 +643,15 @@ void CBActivateField( Widget w, caddr_t client_data, caddr_t call_data)
 	 * If this is the only textfield in this form, submit the form.
 	 */
 	if (cnt == 1)
-		CBSubmitForm(w, client_data, call_data);
+		CBSubmitForm(w, clid, calld);
 }
 
-void CBResetForm( Widget w, caddr_t client_data, caddr_t call_data)
+void CBResetForm( Widget w, caddr_t clid, caddr_t calld)
 {
-	FormInfo *fptr = (FormInfo *)client_data;
+	FormInfo *fptr = (FormInfo *)clid;
 	HTMLWidget hw = (HTMLWidget)(fptr->hw);
 	WidgetInfo *wptr;
 	int widget_count, cnt;
-	XmPushButtonCallbackStruct *pb =
-		(XmPushButtonCallbackStruct *)call_data;
 
 	if (fptr->end == -1)  /* unterminated FORM tag */
 	{
@@ -797,8 +787,6 @@ void CBResetForm( Widget w, caddr_t client_data, caddr_t call_data)
 		case W_OPTIONMENU:
 			if (wptr->value != NULL) {
 				Widget hist = (Widget)wptr->value;
-				Cardinal argcnt;
-				Arg arg[5];
 
 				argcnt = 0;
 				XtSetArg(arg[argcnt], XmNmenuHistory, hist);
@@ -844,7 +832,6 @@ void HideWidgets( HTMLWidget hw)
 	/*
 	 * Make sure all expose events have been dealt with first.
 	 */
-	XmUpdateDisplay((Widget)hw);
 	wptr = hw->html.widget_list;
 	while (wptr != NULL) {
 		if ((wptr->w != NULL)&&(wptr->mapped == True)) {
@@ -855,6 +842,7 @@ void HideWidgets( HTMLWidget hw)
 	}
 				/* Force the exposure events into the queue */
 	XSync(XtDisplay(hw), False);
+	XmUpdateDisplay((Widget)hw);
 				/* Remove all Expose events for the view window */
 	while (XCheckWindowEvent(XtDisplay(hw->html.view),
 		XtWindow(hw->html.view), ExposureMask, &event) == True)
@@ -999,13 +987,11 @@ char ** ParseCommaList( char *str, int *count)
 	if ((str == NULL)||(*str == '\0'))
 		return((char **)NULL);
 	str_copy = (char *)malloc(strlen(str) + 1);
-	if (str_copy == NULL)
-		return((char **)NULL);
+	CHECK_OUT_OF_MEM(str_copy);
 	strcpy(str_copy, str);
 
 	list = (char **)malloc(50 * sizeof(char *));
-	if (list == NULL)
-		return((char **)NULL);
+	CHECK_OUT_OF_MEM(list);
 	max_cnt = 50;
 
 	/*
@@ -1020,8 +1006,7 @@ char ** ParseCommaList( char *str, int *count)
 	while (*tptr != '\0') {
 		if ((cnt + 1) == max_cnt) {
 			tlist = (char **)malloc((max_cnt +50) * sizeof(char *));
-			if (tlist == NULL)
-				return((char **)NULL);
+			CHECK_OUT_OF_MEM(tlist);
 			for (i=0; i<cnt; i++) {
 				tlist[i] = list[i];
 			}
@@ -1031,8 +1016,7 @@ char ** ParseCommaList( char *str, int *count)
 		}
 		*tptr = '\0';
 		list[cnt] = (char *)malloc(strlen(val) + 1);
-		if (list[cnt] == NULL)
-			return((char **)NULL);
+		CHECK_OUT_OF_MEM(list[cnt]);
 		strcpy(list[cnt], val);
 		cnt++;
 
@@ -1040,15 +1024,13 @@ char ** ParseCommaList( char *str, int *count)
 		tptr = NextComma(val);
 	}
 	list[cnt] = (char *)malloc(strlen(val) + 1);
-	if (list[cnt] == NULL)
-		return((char **)NULL);
+	CHECK_OUT_OF_MEM(list[cnt]);
 	strcpy(list[cnt], val);
 	cnt++;
 
 	free(str_copy);
 	tlist = (char **)malloc(cnt * sizeof(char *));
-	if (tlist == NULL)
-		return((char **)NULL);
+	CHECK_OUT_OF_MEM(tlist);
 	for (i=0; i<cnt; i++)
 		tlist[i] = list[i];
 	free((char *)list);
@@ -1228,9 +1210,7 @@ char ** MakeOptionMappings( char **list1, char **list2, int list_cnt)
 	return(list);
 }
 
-/********** MOTIF VERSION *************/
-/*
- * Make the appropriate widget for this tag, and fill in an
+/* Make the appropriate widget for this tag, and fill in an
  * WidgetInfo structure and return it.
  */
 WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
@@ -1315,12 +1295,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			}
 			w = XmCreateToggleButton(hw->html.view, widget_name,
 				arg, argcnt);
-                        if(!hw->html.focus_follows_mouse) {
-                            XtOverrideTranslations(w,
-                                    XtParseTranslationTable(traversal_table));
-                            XtOverrideTranslations(w,
-                                    XtParseTranslationTable("<Btn1Down>: Arm() traversal_current()"));
-                        }
 
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
@@ -1376,9 +1350,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			}
 			w = XmCreateToggleButton(hw->html.view, widget_name,
 				arg, argcnt);
-                        if(!hw->html.focus_follows_mouse)
-                          XtOverrideTranslations(w,
-                                  XtParseTranslationTable(traversal_table));
   
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
@@ -1395,24 +1366,18 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			label = NULL;
 			value = ParseMarkTag(text, MT_INPUT, "VALUE");
 			if ((value == NULL)||(*value == '\0')) {
-				value = (char *)malloc(strlen("Submit Query") +1);
-				strcpy(value, "Submit Query");
+				value = (char *)malloc(strlen("Submit") +1);
+				strcpy(value, "Submit");
 			}
 			argcnt = 0;
 			XtSetArg(arg[argcnt], XmNx, x); argcnt++;
 			XtSetArg(arg[argcnt], XmNy, y); argcnt++;
-			/*XtSetArg(arg[argcnt], XmNnavigationType, XmNONE);
-			argcnt++;*/
-			if (value != NULL) {
-				label = XmStringCreateSimple(value);
-				XtSetArg(arg[argcnt], XmNlabelString, label);
-				argcnt++;
-			}
+/*XtSetArg(arg[argcnt], XmNnavigationType, XmNONE); argcnt++;*/
+			label = XmStringCreateSimple(value);
+			XtSetArg(arg[argcnt], XmNlabelString, label);
+			argcnt++;
 			w = XmCreatePushButton(hw->html.view, widget_name,
 				arg, argcnt);
-                        if(!hw->html.focus_follows_mouse)
-                          XtOverrideTranslations(w,
-                                  XtParseTranslationTable(traversal_table));
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
 			if (label != NULL) {
@@ -1444,9 +1409,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			}
 			w = XmCreatePushButton(hw->html.view, widget_name,
 				arg, argcnt);
-                        if(!hw->html.focus_follows_mouse)
-                          XtOverrideTranslations(w,
-                                  XtParseTranslationTable(traversal_table));
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
 			if (label != NULL)
@@ -1473,9 +1435,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			}
 			w = XmCreatePushButton(hw->html.view, widget_name,
 				arg, argcnt);
-                        if(!hw->html.focus_follows_mouse)
-                          XtOverrideTranslations(w,
-                                  XtParseTranslationTable(traversal_table));
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
 			if (label != NULL) {
@@ -1495,7 +1454,7 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			int return_cnt;
 			char **vlist;
 			int vlist_cnt;
-			int i, mult, size;
+			int i, mult;
 
 			type = -1;
 			tptr = ParseMarkTag(text, MT_INPUT, "HINT");
@@ -1587,10 +1546,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 						bname, arg, argcnt);
 					XtManageChild(button);
 					XmStringFree(label);
-                                        if(!hw->html.focus_follows_mouse)
-                                          XtOverrideTranslations(button,
-                                            XtParseTranslationTable(traversal_table));
-
 					if ((vlist_cnt > 0)&&
 					    (vlist[0] != NULL)&&
 					    (strcmp(vlist[0], list[i]) ==0)) {
@@ -1640,14 +1595,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 				}
 				w = XmCreateOptionMenu(hw->html.view,
 					widget_name, arg, argcnt);
-                                if(!hw->html.focus_follows_mouse)
-                                  {
-                                    XtOverrideTranslations(w,
-                                     XtParseTranslationTable(traversal_table));
-                                    XtOverrideTranslations(pulldown,
-                                     XtParseTranslationTable(traversal_table));
-                                  }
-  
                                 argcnt = 0;
                                 xmstr = XmStringCreateSimple ("");
                                 XtSetArg(arg[argcnt], XmNlabelString,
@@ -1726,9 +1673,6 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 					argcnt++;
 				}
 				w = XmCreateList(scroll, widget_name,arg, argcnt);
-                                if(!hw->html.focus_follows_mouse)
-                                  XtOverrideTranslations(w,
-                                     XtParseTranslationTable(traversal_table));
                                 XtManageChild(w);
 				XtManageChild(w);
 				w = scroll;
@@ -1790,23 +1734,9 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
                         argcnt++;*/
 			w = XmCreateTextField(hw->html.view, widget_name,
 				arg, argcnt);
-			XtOverrideTranslations(w, XtParseTranslationTable(text_translations));
 
-			if(!hw->html.focus_follows_mouse)
-			   XtOverrideTranslations(w, 
-				    XtParseTranslationTable(traversal_table));
-/*
- * The proper order here is XtSetMappedWhenManaged, XtManageChild.  But a bug
- * in some versions of Motif1.1 makes us do it the other way.  All versions
- * of 1.2 should have this fixed
- */
-#ifdef MOTIF1_2
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
-#else
-			XtManageChild(w);
-			XtSetMappedWhenManaged(w, False);
-#endif /* MOTIF1_2 */
 			XtAddCallback(w, XmNactivateCallback,
 				(XtCallbackProc)CBActivateField, (caddr_t)fptr);
 			XtAddCallback(w, XmNmodifyVerifyCallback,
@@ -1814,49 +1744,30 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 		}
 		else if((type_str!=NULL)&&(strcasecmp(type_str, "textarea") ==0))
 		{
-			char **list;
-			int list_cnt;
 			int rows, cols;
 			Widget scroll;
 
 			type = W_TEXTAREA;
 
-			/*
-			 * If there is no SIZE, look for ROWS and COLS
-			 * directly.
-			 * SIZE is COLUMNS,ROWS parse the list
-			 */
-			rows = -1;
-			cols = -1;
-			tptr = ParseMarkTag(text, MT_INPUT, "SIZE");
-			if (tptr == NULL) {
-				tptr = ParseMarkTag(text, MT_INPUT, "ROWS");
-				if (tptr != NULL) {
-					rows = atoi(tptr);
-					free(tptr);
-				}
-				tptr = ParseMarkTag(text, MT_INPUT, "COLS");
-				if (tptr != NULL) {
-					cols = atoi(tptr);
-					free(tptr);
-				}
-			} else {
-				list = ParseCommaList(tptr, &list_cnt);
+			/* look for ROWS and COLS directly. */
+			rows = 4;
+			cols = 40;
+			tptr = ParseMarkTag(text, MT_INPUT, "ROWS");
+			if (tptr != NULL) {
+				rows = atoi(tptr);
 				free(tptr);
-
-				if (list_cnt == 1) {
-					cols = atoi(list[0]);
-				}
-				else if (list_cnt > 1)
-				{
-					cols = atoi(list[0]);
-					rows = atoi(list[1]);
-				}
-				FreeCommaList(list, list_cnt);
 			}
+			if ( rows <= 0)
+				rows = 4;
+			tptr = ParseMarkTag(text, MT_INPUT, "COLS");
+			if (tptr != NULL) {
+				cols = atoi(tptr);
+				free(tptr);
+			}
+			if (cols <=0)
+				cols = 40;
 
-			/*
-			 * Grab the starting value of the text here.
+			/* Grab the starting value of the text here.
 			 * NULL if none.
 			 */
 			value = ParseMarkTag(text, MT_INPUT, "VALUE");
@@ -1871,173 +1782,61 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			argcnt = 0;
 			XtSetArg(arg[argcnt], XmNeditMode, XmMULTI_LINE_EDIT);
 			argcnt++;
-			if (cols > 0) {
-				XtSetArg(arg[argcnt], XmNcolumns, cols);
-				argcnt++;
-			}
-			if (rows > 0) {
-				XtSetArg(arg[argcnt], XmNrows, rows);
-				argcnt++;
-			}
+			XtSetArg(arg[argcnt], XmNcolumns, cols); argcnt++;
+			XtSetArg(arg[argcnt], XmNrows, rows); argcnt++;
 			if (value != NULL) {
 				XtSetArg(arg[argcnt], XmNvalue, value);
 				argcnt++;
 			}
-                        /*XtSetArg(arg[argcnt], XmNnavigationType, XmNONE);
-                        argcnt++;*/ 
+/*XtSetArg(arg[argcnt], XmNnavigationType, XmNONE); argcnt++;*/ 
 			w = XmCreateText(scroll, widget_name, arg, argcnt);
 			XtManageChild(w);
-                        XtOverrideTranslations(w, XtParseTranslationTable(text_translations));                           
-                                
-                        if(!hw->html.focus_follows_mouse)
-			  XtOverrideTranslations(w, 
-				     XtParseTranslationTable(traversal_table));
 			w = scroll;
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
 		}
-		else /* if no type, assume type=text */
+		else /* if no type, assume type=text . Single line text field */
 		{
-			char **list;
-			int list_cnt;
-			int rows, cols;
-			Widget scroll;
+			int cols=40;
 
-			/*
-			 * SIZE can be either COLUMNS or COLUMNS,ROWS
-			 * we assume COLUMNS,ROWS and parse the list
-			 */
+			/* SIZE can be  COLUMNS, assume a TEXTFIELD */
+			type = W_TEXTFIELD;
 			tptr = ParseMarkTag(text, MT_INPUT, "SIZE");
-			list = ParseCommaList(tptr, &list_cnt);
-			if (tptr != NULL)
+			if (tptr != NULL){
+				cols = atoi(tptr);
 				free(tptr);
-
-			/*
-			 * If only COLUMNS specified, or SIZE not specified
-			 * assume a TEXTFIELD
-			 * Otherwise a TEXTAREA.
-			 */
-			if (list_cnt <= 1) {
-				type = W_TEXTFIELD;
-				if (list_cnt == 1) {
-					cols = atoi(list[0]);
-				} else {
-					cols = -1;
-				}
-			} else {
-				type = W_TEXTAREA;
-				cols = atoi(list[0]);
-				rows = atoi(list[1]);
-				       /* be a textfield if only one row */
-                                if(rows==1)
-                                  type=W_TEXTFIELD;
 			}
-			/*
-			 * Now that we have cols, and maybe rows, free the list
-			 */
-			FreeCommaList(list, list_cnt);
-
-			/*
-			 * Grab the starting value of the text here.
-			 * NULL if none.
-			 */
+			/* Grab the starting value of text here. NULL if none. */
 			value = ParseMarkTag(text, MT_INPUT, "VALUE");
 
-			/*
-			 * For textfileds parse maxlength and
-			 * set up the widget.
-			 */
-			if (type == W_TEXTFIELD) {
-				maxlength = -1;
-				tptr = ParseMarkTag(text, MT_INPUT,"MAXLENGTH");
-				if (tptr != NULL) {
-					maxlength = atoi(tptr);
-					free(tptr);
-				}
-				argcnt = 0;
-				XtSetArg(arg[argcnt], XmNx, x); argcnt++;
-				XtSetArg(arg[argcnt], XmNy, y); argcnt++;
-				if (cols > 0) {
-					XtSetArg(arg[argcnt], XmNcolumns, cols);
-					argcnt++;
-				}
-				if (maxlength > 0) {
-					XtSetArg(arg[argcnt], XmNmaxLength,
-						maxlength);
-					argcnt++;
-				}
-				if (value != NULL) {
-					XtSetArg(arg[argcnt], XmNvalue, value);
-					argcnt++;
-				}
-                                /*XtSetArg(arg[argcnt], XmNnavigationType, XmNONE);                      
-                                argcnt++;*/
+			/* parse maxlength and set up the widget. */
+			maxlength = -1;
+			tptr = ParseMarkTag(text, MT_INPUT,"MAXLENGTH");
+			if (tptr != NULL) {
+				maxlength = atoi(tptr);
+				free(tptr);
+			}
+			argcnt = 0;
+			XtSetArg(arg[argcnt], XmNx, x); argcnt++;
+			XtSetArg(arg[argcnt], XmNy, y); argcnt++;
+			XtSetArg(arg[argcnt], XmNcolumns, cols); argcnt++;
+			if (maxlength > 0) {
+				XtSetArg(arg[argcnt], XmNmaxLength, maxlength);
+				argcnt++;
+			}
+			if (value != NULL) {
+				XtSetArg(arg[argcnt], XmNvalue, value); argcnt++;
+			}
+/*XtSetArg(arg[argcnt], XmNnavigationType, XmNONE); argcnt++;*/
 
-				w = XmCreateTextField(hw->html.view,
+			w = XmCreateTextField(hw->html.view,
 					widget_name, arg, argcnt);
-				
-                                XtOverrideTranslations(w, XtParseTranslationTable(text_translations));
-                                if(!hw->html.focus_follows_mouse)
-				   XtOverrideTranslations(w, 
-				     XtParseTranslationTable(traversal_table));
-			} else {
-			/*
-			 * Else this is a TEXTAREA.  Maxlength is ignored,
-			 * and we set up the scrolled window
-			 */
-				argcnt = 0;
-				XtSetArg(arg[argcnt], XmNx, x); argcnt++;
-				XtSetArg(arg[argcnt], XmNy, y); argcnt++;
-				scroll = XmCreateScrolledWindow(hw->html.view,
-					"Scroll", arg, argcnt);
-
-				argcnt = 0;
-				XtSetArg(arg[argcnt], XmNeditMode,
-					XmMULTI_LINE_EDIT);
-				argcnt++;
-				if (cols > 0) {
-					XtSetArg(arg[argcnt], XmNcolumns, cols);
-					argcnt++;
-				}
-				if (rows > 0) {
-					XtSetArg(arg[argcnt], XmNrows, rows);
-					argcnt++;
-				}
-				if (value != NULL) {
-					XtSetArg(arg[argcnt], XmNvalue, value);
-					argcnt++;
-				}
-				XtSetArg(arg[argcnt], XmNnavigationType, XmNONE);
-				argcnt++;
-				w = XmCreateText(scroll, widget_name,arg, argcnt);
-				XtManageChild(w);
-                                XtOverrideTranslations(w, XtParseTranslationTable(text_translations));
-                                if(!hw->html.focus_follows_mouse)
-                                  XtOverrideTranslations(w,
-                                     XtParseTranslationTable(traversal_table));
-				w = scroll;
-			}
-
-/*
- * The proper order here is XtSetMappedWhenManaged, XtManageChild.  But a bug
- * in some versions of Motif1.1 makes us do it the other way.  All versions
- * of 1.2 should have this fixed
- */
-#ifdef MOTIF1_2
 			XtSetMappedWhenManaged(w, False);
 			XtManageChild(w);
-#else
-			XtManageChild(w);
-			XtSetMappedWhenManaged(w, False);
-#endif /* MOTIF1_2 */
 
-			/*
-			 * For textfields, a CR might be an activate
-			 */
-			if (type == W_TEXTFIELD) {
-				XtAddCallback(w, XmNactivateCallback,
-					(XtCallbackProc)CBActivateField, (caddr_t)fptr);
-			}
+			/* For textfields, a CR might be an activate */
+			XtAddCallback(w, XmNactivateCallback,
+				(XtCallbackProc)CBActivateField, (caddr_t)fptr);
 		}
 		if (type_str != NULL)
 			free(type_str);
@@ -2053,9 +1852,9 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			XtGetValues(w, arg, argcnt);
 			/* Set it to default so we don't lose it on "back"*/
                         XtVaSetValues(w,
-                          XmNbackground, hw->html.background_SAVE,
-                          XmNtopShadowColor, hw->html.top_color_SAVE,
-                          XmNbottomShadowColor, hw->html.bottom_color_SAVE,
+                          XmNbackground, hw->html.def_res.bgcolor,
+                          XmNtopShadowColor, hw->html.def_res.top_color,
+                          XmNbottomShadowColor, hw->html.def_res.bottom_color,
                           NULL);
 		} else {
 			width = 0;
@@ -2083,9 +1882,9 @@ WidgetInfo * MakeWidget( HTMLWidget hw, char *text,
 			XtSetValues(wlist->w, arg, argcnt);
 			/* Set it to default so we don't lose it on "back"*/
                         XtVaSetValues(wlist->w,
-                          XmNbackground, hw->html.background_SAVE,
-                          XmNtopShadowColor, hw->html.top_color_SAVE,
-                          XmNbottomShadowColor, hw->html.bottom_color_SAVE,
+                          XmNbackground, hw->html.def_res.bgcolor,
+                          XmNtopShadowColor, hw->html.def_res.top_color,
+                          XmNbottomShadowColor, hw->html.def_res.bottom_color,
                           NULL);
 		}
 
@@ -2111,203 +1910,6 @@ unsigned long bp=BlackPixel(XtDisplay(hw),DefaultScreen(XtDisplay(hw)));
 	}
 }
 
-void traversal_forward(Widget w, XEvent * event,
-	       String * params, Cardinal * num_params)
-{ 
-  HTMLTraverseTabGroups(w, XmTRAVERSE_NEXT_TAB_GROUP);
-}
-
-void traversal_back(Widget w, XEvent * event,
-	       String * params, Cardinal * num_params)
-{ 
-  HTMLTraverseTabGroups(w, XmTRAVERSE_PREV_TAB_GROUP);
-}
-void traversal_current(Widget w, XEvent *event,
-               String *params, Cardinal *num_params)
-{       
-  HTMLTraverseTabGroups(w, XmTRAVERSE_CURRENT);
-}
-
-void traversal_end(Widget w, XEvent * event,
-	       String * params, Cardinal * num_params)
-{
-  Widget top;
-  HTMLWidget hw = (HTMLWidget) w;
-  int i=0;
-
-  while(i < 5)
-    {
-      if(XtClass((Widget) hw) != htmlWidgetClass)
-        hw = (HTMLWidget) XtParent((Widget) hw);
-      else
-        break;
-      i++;
-    }
-
-  top = (Widget) hw;
-  while(!XtIsTopLevelShell(top))
-    top = XtParent(top);
-
-  if(XtClass((Widget) hw) != htmlWidgetClass)
-    {
-      fprintf(stderr, "Error in traversal_end action.");
-      return;
-    }
-
-  if(hw->html.focus_follows_mouse)
-    return;
-  else
-    { 
-      XtSetKeyboardFocus(top, hw->html.view);
-      HTMLTraverseTabGroups(w, XmTRAVERSE_HOME);
-    }
-}
-
-/* makes it possible to traverse forms */
-void HTMLTraverseTabGroups(Widget w, int how)
-{
-  static WidgetInfo *lptr;
-  Widget top;
-  Boolean ret;
-  int i=0;
-  HTMLWidget hw = (HTMLWidget) w;
-
-  /* due to the generality of this function the HTMLwidget could be anywhere */
-  for(i=0;i<4;i++)
-    {
-      if((hw != NULL) && (XtClass((Widget) hw) != htmlWidgetClass))
-        hw = (HTMLWidget) XtParent((Widget) hw);
-      else
-        break;
-    }
-
-  if(!hw || XtClass((Widget) hw) != htmlWidgetClass)
-    return;
-
-  /* make sure we have business to do */
-  if(!hw->html.widget_list || hw->html.focus_follows_mouse)
-    return;
-
-  top = (Widget) hw;
-  while(!XtIsTopLevelShell(top))
-    top = XtParent(top);
-
-  switch(how)
-    {
-    case XmTRAVERSE_NEXT_TAB_GROUP:
-      if(!lptr)
-        lptr=hw->html.widget_list;
-      else if(lptr->next && lptr->next->w && XtIsManaged(lptr->next->w))
-        lptr = lptr->next;
-        
-      /* Patch for hidden fields... SWP */
-      while (lptr && !lptr->w) {
-        if (lptr->next) {
-          lptr=lptr->next;
-        }
-        else {
-          lptr=NULL;
-        }
-      } 
-
-      if (!lptr) {
-        break;
-      } 
-
-      /* automagickly scroll */
-      if(XtIsManaged(hw->html.vbar) &&
-         (lptr->y > (hw->html.view_height+hw->html.scroll_y-10)))
-        {
-          int val, ss, in, pg_in, amount;
-        
-          amount = lptr->y-hw->html.view_height/2;
-          if(amount<0)
-            amount=0;
-          XmScrollBarGetValues(hw->html.vbar, &val, &ss, &in, &pg_in);
-        
-          if(amount > (hw->html.doc_height-ss-5))
-            amount = hw->html.doc_height-ss-5;
-        
-          XmScrollBarSetValues(hw->html.vbar, amount,
-                               ss, in, pg_in,True);
-        }
-        
-      if(XtClass(lptr->w) == xmScrolledWindowWidgetClass)
-        {
-          Widget text;
-          XtVaGetValues(lptr->w, XmNworkWindow, &text, NULL);
-          XtSetKeyboardFocus(top, text);
-        }
-      else
-        XtSetKeyboardFocus(top, lptr->w);
-      break;
-    case XmTRAVERSE_PREV_TAB_GROUP:
-      if(!lptr)
-        lptr=hw->html.widget_list;
-      else if(lptr->prev)
-        lptr = lptr->prev;
-        
-      /* Patch for hidden fields... SWP */
-      while (lptr && !lptr->w) {
-        if (lptr->prev) {
-          lptr=lptr->prev;
-        }
-        else {
-          lptr=NULL;
-        }
-      } 
-
-      if (!lptr) {
-        break;
-      } 
-
-      /* automagickly scroll */
-      if(XtIsManaged(hw->html.vbar) &&
-         (lptr->y < hw->html.scroll_y+10))
-        {
-          int val, ss, in, pg_in, amount;
-        
-          XmScrollBarGetValues(hw->html.vbar, &val, &ss, &in, &pg_in);
-        
-          amount = lptr->y - hw->html.view_height/2;
-        
-          if(amount<0)
-            amount=0;
-        
-          XmScrollBarSetValues(hw->html.vbar, amount,
-                               ss, in, pg_in,True);
-        }
-        
-      if(XtClass(lptr->w) == xmScrolledWindowWidgetClass)
-        {
-          Widget text;
-        
-          XtVaGetValues(lptr->w, XmNworkWindow, &text, NULL);
-          XtSetKeyboardFocus(top, text);
-        }
-      else
-        XtSetKeyboardFocus(top, lptr->w);
-      break;
-    case XmTRAVERSE_HOME:
-      lptr=NULL;
-      break;
-    case XmTRAVERSE_CURRENT:
-      lptr = hw->html.widget_list;
-
-      /* check parent to allow for text areas (lptr->w would be scroll) */
-      while(lptr!=NULL)
-        {
-          if((lptr->w == w) || (lptr->w == XtParent(w)))
-            break;
-          lptr = lptr->next;
-        }
-        
-      XtSetKeyboardFocus(top, w);
-
-      break;
-    }
-}
-
 /* Place a Widget. Add an element record for it.  */
 void WidgetPlace(HTMLWidget hw, struct mark_up *mptr, PhotoComposeContext *pcc)
 {
@@ -2350,3 +1952,30 @@ void WidgetPlace(HTMLWidget hw, struct mark_up *mptr, PhotoComposeContext *pcc)
         pcc->x = pcc->x + width ;      
         pcc->is_bol = False;
 }
+
+void HTMLFreeWidgetInfo(void *ptr)
+{
+	WidgetInfo *wptr = (WidgetInfo *)ptr;
+	WidgetInfo *tptr;
+
+	while (wptr != NULL) {
+		tptr = wptr;
+		wptr = wptr->next;
+		if (tptr->w != NULL) {
+/* This is REALLY DUMB, but X generates an expose event
+* for the destruction of the Widgte, even if it isn't
+* mapped at the time it is destroyed.
+* So I move the invisible widget to -1000,-1000
+* before destroying it, to avoid a visible flash.
+*/     
+			XtMoveWidget(tptr->w, -1000, -1000);
+			XtDestroyWidget(tptr->w);
+		}
+		if (tptr->name != NULL)
+			free(tptr->name);
+		if ((tptr->value != NULL)&&(tptr->type != W_OPTIONMENU))
+			free(tptr->value);
+		free((char *)tptr);
+	}
+}
+
