@@ -1,25 +1,28 @@
 /* Please read copyright.ncsa. Don't remove next line */
-#include "copyright.ncsa"
+#include "../Copyrights/copyright.ncsa"
 
 /* Interface for mailto: URLs, stolen from techsupport.c */
 
+#include <assert.h>
+
+#include "../libnut/url-utils.h"
 #include "../libhtmlw/HTML.h"
 #include "mosaic.h"
 
-#include "../libnut/url-utils.h"
+#include "gui.h"
 #include "gui-dialogs.h"
+#include "mime.h"
+#include "paf.h"
+#include "mailto.h"
+#include "URLParse.h"
 
-#ifndef DISABLE_TRACE
-extern int srcTrace;
+#ifdef DEBUG
+#define DEBUG_GUI
 #endif
 
-extern int do_post;
-extern char *post_content_type;
-extern char *post_data;
-extern char pre_title[80];
+char *post_content_type;
 
-extern mo_window *current_win;
-mo_status mo_post_mailto_form_win (char *to_address, char *subject);
+static mo_status mo_post_mailto_form_win (mo_window *win,char *to_address, char *subject);
 mo_status mo_send_mailto_message (char *text, char *to, char *subj, 
 				  char *content_type, char *url);
 void do_mailto_post(mo_window *win, char *to, char *from, char *subject, 
@@ -51,7 +54,7 @@ mo_window *win = (mo_window*)client_data;
         char *buf, *final, tmpbuf[80];
 	int final_len;
 
-        buf=my_strerror(errno);
+        buf=strerror(errno);
         if (!buf || !*buf || !strcmp(buf,"Error 0")) {
                 sprintf(tmpbuf,"Unknown Error");
                 buf=tmpbuf;
@@ -100,15 +103,15 @@ static XmxCallback (mailto_win_cb0)		/* send */
 	mo_window *win = (mo_window*)client_data;
 	char *msg, *subj, *to;
       
-	XtUnmanageChild (win->mailto_win);
-	msg = XmxTextGetString (win->mailto_text);
+	XtPopdown (XtParent(win->mailto_win));
+	msg = XmTextGetString (win->mailto_text);
 	if (!msg)
 		return;
 	if (msg[0] == '\0')
 		return;
-	to = XmxTextGetString (win->mailto_tofield);
-	subj = XmxTextGetString (win->mailto_subfield);
-	mo_send_mailto_message(msg,to,subj,"text/plain",win->current_node->url);
+	to = XmTextGetString (win->mailto_tofield);
+	subj = XmTextGetString (win->mailto_subfield);
+	mo_send_mailto_message(msg,to,subj,"text/plain",win->current_node->aurl);
 	free (msg);
 	free (to);
 	free (subj);
@@ -117,15 +120,21 @@ static XmxCallback (mailto_win_cb1)		/* dismiss */
 {
 	mo_window *win = (mo_window*)client_data;
 
-	XtUnmanageChild (win->mailto_win); 
+	XtPopdown (XtParent(win->mailto_win)); 
 }
 static XmxCallback (mailto_win_cb2)		/* help */
 {
 	mo_window *win = (mo_window*)client_data;
+	mo_window * neww;
+	RequestDataStruct rds;
 
-	mo_open_another_window (win, 
-		mo_assemble_help_url ("help-on-mailto.html"),
-		NULL, NULL);
+	rds.ct = rds.post_data = NULL;
+	rds.is_reloading = False;
+	rds.req_url = mo_assemble_help_url ("help-on-mailto.html");
+	rds.gui_action = HTML_LOAD_CALLBACK;
+	neww = mo_make_window(win,MC_MO_TYPE_UNICAST);
+		neww->navigation_action = NAVIGATE_NEW;
+	MMPafLoadHTMLDocInWin (neww, &rds);
 }
 static XmxCallback (mailto_win_cb3)		/* insert file */
 {
@@ -133,7 +142,7 @@ static XmxCallback (mailto_win_cb3)		/* insert file */
       
 	if (!win->mail_fsb_win) {
 		win->mail_fsb_win = XmxMakeFileSBDialog (win->mailto_win,
-			"NCSA Mosaic: Include File In Mail Message",
+			"mMosaic: Include File In Mail Message",
 			"Name of file to include:",
 			include_fsb_cb, (XtPointer)win);
 	} else {
@@ -147,14 +156,14 @@ static XmxCallback (mailto_win_cb4)		/* insert url */
 	mo_window *win = (mo_window*)client_data;
 	long pos;
 
-        if(win->current_node->url){   
+        if(win->current_node->aurl){   
             XmTextInsert (win->mailto_text,
                           pos = XmTextGetInsertionPosition (win->mailto_text),
-                          win->current_node->url);
+                          win->current_node->aurl);
                 /* move insertion position to past this line
                    to avoid inserting the lines in reverse order */
             XmTextSetInsertionPosition (win->mailto_text,
-                                        pos + strlen(win->current_node->url));
+                                        pos + strlen(win->current_node->aurl));
         }  
 }
 
@@ -165,9 +174,9 @@ static XmxCallback (mailto_form_win_cb0) 		/* send */
 	char *subj, *to, *namestr;
 
 	XtUnmanageChild (win->mailto_form_win);
-	to = XmxTextGetString (win->mailto_form_tofield);
-	subj = XmxTextGetString (win->mailto_form_subfield);
-	namestr = XmxTextGetString(win->mailto_form_fromfield);
+	to = XmTextGetString (win->mailto_form_tofield);
+	subj = XmTextGetString (win->mailto_form_subfield);
+	namestr = XmTextGetString(win->mailto_form_fromfield);
 	do_mailto_post(win,to,namestr,subj,win->post_data);
 	free (namestr);
 	free (to);
@@ -191,48 +200,50 @@ static XmxCallback (mailto_form_win_cb1)		/* dismiss */
 
 static XmxCallback (mailto_form_win_cb2)		/* help */
 {
-	mo_window *win =  (mo_window*)client_data;
+	mo_window *win = (mo_window*)client_data;
+	mo_window * neww;
+	RequestDataStruct rds;
 
-	mo_open_another_window (win, 
-		mo_assemble_help_url ("help-on-mailto-form.html"),
-		NULL, NULL);
+	rds.ct = rds.post_data = NULL;
+	rds.is_reloading = False;
+	rds.req_url = mo_assemble_help_url ("help-on-mailto-form.html");
+	rds.gui_action = HTML_LOAD_CALLBACK;
+	neww = mo_make_window(win,MC_MO_TYPE_UNICAST);
+	neww->navigation_action = NAVIGATE_NEW;
+	MMPafLoadHTMLDocInWin (neww, &rds);
 }
 
-mo_status mo_post_mailto_win (char *to_address, char *subject)
+mo_status mo_post_mailto_win (mo_window *win, char *to_address, char *subject)
 {
-  mo_window *win = current_win;
-  FILE *fp;
-  long pos;
-  char namestr[1024],tmp[1024];
+	FILE *fp;
+	long pos;
+	char namestr[1024],tmp[1024];
 
-  if (do_post) {
-	if (!subject || !*subject) {
-		char str[BUFSIZ];
+	if (win->post_data) {
+		fprintf(stderr, "Bug, please report...\n");
+		fprintf(stderr,"mo_post_mailto_win: win with post_data\n");
+		assert(0);
+		if (!subject || !*subject) {
+			char str[BUFSIZ];
 
-		sprintf(str,"Form Result(s) Posted from %s",pre_title);
-
-		return(mo_post_mailto_form_win(to_address,str));
+			sprintf(str,"Form Result(s) Posted from %s","");
+			return(mo_post_mailto_form_win(win,to_address,str));
+		} else {
+			return(mo_post_mailto_form_win(win,to_address,subject));
+		}
 	}
-	else {
-		return(mo_post_mailto_form_win(to_address,subject));
-	}
-  }
-
-  if (!win->mailto_win)
-    {
-      Widget dialog_frame;
-      Widget dialog_sep, buttons_form;
-      Widget mailto_form;
-      Widget tolabel, sublabel, fromlabel;
+	if (!win->mailto_win) {
+		Widget dialog_frame;
+		Widget dialog_sep, buttons_form;
+		Widget mailto_form;
+		Widget tolabel, sublabel, fromlabel;
       
       /* Create it for the first time. */
-      win->mailto_win = XmxMakeFormDialog (win->base, 
-		"NCSA Mosaic: Mail To Author");
+      win->mailto_win = XmxMakeFormDialog (win->base, "mMosaic: Mail To Author");
       dialog_frame = XmxMakeFrame (win->mailto_win, XmxShadowOut);
       
       /* Constraints for base. */
-      XmxSetConstraints 
-        (dialog_frame, XmATTACH_FORM, XmATTACH_FORM, 
+      XmxSetConstraints (dialog_frame, XmATTACH_FORM, XmATTACH_FORM, 
          XmATTACH_FORM, XmATTACH_FORM, NULL, NULL, NULL, NULL);
       
       /* Main form. */
@@ -264,28 +275,23 @@ mo_status mo_post_mailto_win (char *to_address, char *subject)
 
       /* constraints for FROM */
       XmxSetOffsets(fromlabel, 14, 10, 10, 10);
-      XmxSetConstraints
-	(fromlabel, XmATTACH_FORM, XmATTACH_NONE, XmATTACH_FORM, 
+      XmxSetConstraints (fromlabel, XmATTACH_FORM, XmATTACH_NONE, XmATTACH_FORM, 
 	 XmATTACH_NONE, NULL, NULL, NULL, NULL);
       XmxSetOffsets(win->mailto_fromfield, 10, 10, 10, 10);
-      XmxSetConstraints
-	(win->mailto_fromfield, XmATTACH_FORM, XmATTACH_NONE, XmATTACH_WIDGET,
+      XmxSetConstraints (win->mailto_fromfield, XmATTACH_FORM, XmATTACH_NONE, XmATTACH_WIDGET,
 	 XmATTACH_FORM, NULL, NULL, fromlabel, NULL);
 
       /* constraints for TO */
       XmxSetOffsets(tolabel, 14, 10, 10, 10);
-      XmxSetConstraints
-	(tolabel, XmATTACH_WIDGET, XmATTACH_NONE, XmATTACH_FORM, XmATTACH_NONE,
+      XmxSetConstraints (tolabel, XmATTACH_WIDGET, XmATTACH_NONE, XmATTACH_FORM, XmATTACH_NONE,
 	 win->mailto_fromfield, NULL, NULL, NULL);
       XmxSetOffsets(win->mailto_tofield, 10, 10, 10, 10);
-      XmxSetConstraints
-	(win->mailto_tofield, XmATTACH_WIDGET, XmATTACH_NONE, XmATTACH_WIDGET, 
+      XmxSetConstraints (win->mailto_tofield, XmATTACH_WIDGET, XmATTACH_NONE, XmATTACH_WIDGET, 
 	 XmATTACH_FORM, win->mailto_fromfield, NULL, tolabel, NULL);
 
       /* constraints for SUBJECT */
       XmxSetOffsets(sublabel, 14, 10, 10, 10);
-      XmxSetConstraints
-	(sublabel, XmATTACH_WIDGET, XmATTACH_NONE, XmATTACH_FORM, 
+      XmxSetConstraints (sublabel, XmATTACH_WIDGET, XmATTACH_NONE, XmATTACH_FORM, 
 	 XmATTACH_NONE, win->mailto_tofield, NULL, NULL, NULL);
       XmxSetOffsets(win->mailto_subfield, 10, 10, 10, 10);
       XmxSetConstraints (win->mailto_subfield, XmATTACH_WIDGET, XmATTACH_NONE, 
@@ -300,46 +306,42 @@ mo_status mo_post_mailto_win (char *to_address, char *subject)
 		(XtPointer)win);
 
       XmxSetOffsets (XtParent (win->mailto_text), 3, 0, 3, 3);
-      XmxSetConstraints
-        (XtParent (win->mailto_text), XmATTACH_WIDGET, XmATTACH_WIDGET, 
+      XmxSetConstraints (XtParent (win->mailto_text), XmATTACH_WIDGET, XmATTACH_WIDGET, 
          XmATTACH_FORM, XmATTACH_FORM,
          win->mailto_subfield, dialog_sep, NULL, NULL);
 
       XmxSetArg (XmNtopOffset, 10);
-      XmxSetConstraints 
-        (dialog_sep, XmATTACH_NONE, XmATTACH_WIDGET, XmATTACH_FORM, 
+      XmxSetConstraints (dialog_sep, XmATTACH_NONE, XmATTACH_WIDGET, XmATTACH_FORM, 
          XmATTACH_FORM,
          NULL, buttons_form, NULL, NULL);
-      XmxSetConstraints 
-        (buttons_form, XmATTACH_NONE, XmATTACH_FORM, XmATTACH_FORM, 
+      XmxSetConstraints (buttons_form, XmATTACH_NONE, XmATTACH_FORM, XmATTACH_FORM, 
 	 XmATTACH_FORM, NULL, NULL, NULL, NULL);
     }
 
   /* fill in text fields */
 
-  sprintf(namestr, "%s <%s>", get_pref_string(eDEFAULT_AUTHOR_NAME),
-          get_pref_string(eDEFAULT_AUTHOR_EMAIL));
+  sprintf(namestr, "%s <%s>", mMosaicAppData.author_full_name,
+          mMosaicAppData.author_email);
   
   XmxTextSetString (win->mailto_fromfield, namestr);
   XmxTextSetString (win->mailto_tofield, to_address);
   if (!subject || !*subject) {
 	char str[BUFSIZ];
 
-	sprintf(str,"Mail from %s",pre_title);
+	sprintf(str,"Mail from %s","");
   	XmTextFieldSetString(win->mailto_subfield,str);
-  }
-  else {
+  } else {
   	XmTextFieldSetString(win->mailto_subfield,subject);
   }
 
   XmxTextSetString (win->mailto_text, "");
   
       /* tack signature on the end if it exists - code from Martin Hamilton */
-  if (get_pref_string(eSIGNATURE)) {
+  if (mMosaicAppData.signature) {
       XmxTextSetString (win->mailto_text, "\n\n");
           /* leave a gap... */
       XmTextSetInsertionPosition (win->mailto_text, 2);
-      if ((fp = fopen(get_pref_string(eSIGNATURE), "r")) != NULL) {
+      if ((fp = fopen(mMosaicAppData.signature, "r")) != NULL) {
           while(fgets(tmp, sizeof(tmp) - 1, fp)) {
               XmTextInsert(win->mailto_text,
                            pos = XmTextGetInsertionPosition (win->mailto_text),
@@ -355,7 +357,7 @@ mo_status mo_post_mailto_win (char *to_address, char *subject)
   XmTextSetInsertionPosition (win->mailto_text, 0);
       
   XmxManageRemanage (win->mailto_win);
-
+  XtPopup(XtParent(win->mailto_win),XtGrabNone);
   return mo_succeed;
 }
 
@@ -401,8 +403,7 @@ char *b=NULL;
 	sprintf(buf,"\n");
 	for(; (b && *b) ;) {
 		val = makeword(b,'&');
-		plustospace(val);
-		unescape_url(val);
+		val = UnEscapeUrl(val);
 		name = makeword(val,'=');
 		if (condense) { /*take out all "multiple isspace()"*/
 			eatSpace(val);
@@ -424,14 +425,10 @@ char *b=NULL;
 }
 
 
-mo_status mo_post_mailto_form_win (char *to_address, char *subject)
+static mo_status mo_post_mailto_form_win (mo_window *win,char *to_address, char *subject)
 {
-  mo_window *win = current_win;
   char namestr[1024],*buf=NULL;
 
-  if (!do_post)
-	return(mo_fail);
-
   if (!win->mailto_form_win) {
       Widget dialog_frame;
       Widget dialog_sep, buttons_form;
@@ -440,7 +437,7 @@ mo_status mo_post_mailto_form_win (char *to_address, char *subject)
       
       /* Create it for the first time. */
       win->mailto_form_win = XmxMakeFormDialog (win->base, 
-			"NCSA Mosaic: Mail Form Results To Author");
+			"mMosaic: Mail Form Results To Author");
       dialog_frame = XmxMakeFrame (win->mailto_form_win, XmxShadowOut);
       
       /* Constraints for base. */
@@ -531,8 +528,8 @@ mo_status mo_post_mailto_form_win (char *to_address, char *subject)
 
   /* fill in text fields */
 
-  sprintf(namestr, "%s <%s>", get_pref_string(eDEFAULT_AUTHOR_NAME),
-          get_pref_string(eDEFAULT_AUTHOR_EMAIL));
+  sprintf(namestr, "%s <%s>", mMosaicAppData.author_full_name,
+          mMosaicAppData.author_email);
   
   XmxTextSetString (win->mailto_form_fromfield, namestr);
   XmxTextSetString (win->mailto_form_tofield, to_address);
@@ -541,9 +538,9 @@ mo_status mo_post_mailto_form_win (char *to_address, char *subject)
   else
   	XmTextFieldSetString(win->mailto_form_subfield,"");
 
-  win->post_data=strdup(post_data);
+/*  win->post_data=strdup(post_data); */
 
-  buf=makeReadable(post_data,1);
+  buf=makeReadable(win->post_data,1);
   XmTextSetString (win->mailto_form_text, buf);
   if (buf)
 	free(buf);
@@ -555,8 +552,8 @@ mo_status mo_post_mailto_form_win (char *to_address, char *subject)
 /* SWP -- 11.15.95 -- ACTION=mailto support for Forms */
 void do_mailto_post(mo_window *win, char *to, char *from, char *subject, char *body) {
 
-	if (!win || !win->current_node || !win->current_node->url ||
-	    !*(win->current_node->url) || !to || !*to || !from || !*from ||
+	if (!win || !win->current_node ||
+	    !to || !*to || !from || !*from ||
 	    !body || !*body || !subject || !*subject) {
 		return;
 	}
@@ -565,24 +562,24 @@ void do_mailto_post(mo_window *win, char *to, char *from, char *subject, char *b
 		char *buf=NULL;
 
 		buf=makeReadable(body,0);
-#ifndef DISABLE_TRACE
-		if (srcTrace) {
+#ifdef DEBUG_GUI
+		if (mMosaicSrcTrace) {
 			fprintf(stderr,"To: [%s]\nFrom: [%s]\nSubj: [%s]\nBody: [%s]\n",to,from,subject,buf);
 		}
 #endif
 		mo_send_mailto_message(buf, to, subject, post_content_type, 
-				       win->current_node->url);
+				       win->current_node->base_url);
 		if (buf)
 			free(buf);
 	} else {
-#ifndef DISABLE_TRACE
-		if (srcTrace) {
+#ifdef DEBUG_GUI
+		if (mMosaicSrcTrace) {
 			fprintf(stderr,"To: [%s]\nFrom: [%s]\nSubj: [%s]\nBody: [%s]\n",to,from,subject,body);
 		}
 #endif
 
 		mo_send_mailto_message(body, to, subject, post_content_type, 
-				       win->current_node->url);
+				       win->current_node->base_url);
 	}
 }
 
@@ -599,11 +596,11 @@ FILE *mo_start_sending_mailto_message (char *to, char *subj,
   if (!to)
     return NULL;
   
-  if (get_pref_string(eMAIL_FILTER_COMMAND)) {
-      sprintf (cmd, "%s | %s", get_pref_string(eMAIL_FILTER_COMMAND), 
-               get_pref_string(eSENDMAIL_COMMAND));
+  if (mMosaicAppData.mail_filter_command) {
+      sprintf (cmd, "%s | %s", mMosaicAppData.mail_filter_command, 
+               mMosaicAppData.sendmail_command);
     } else {
-      sprintf (cmd, "%s", get_pref_string(eSENDMAIL_COMMAND));
+      sprintf (cmd, "%s", mMosaicAppData.sendmail_command);
     }
 
   if ((_fp = popen (cmd, "w")) == NULL)
@@ -613,7 +610,7 @@ FILE *mo_start_sending_mailto_message (char *to, char *subj,
   fprintf (_fp, "Subject: %s\n", subj);
   fprintf (_fp, "Content-Type: %s\n", content_type);
   fprintf (_fp, "Mime-Version: 1.0\n");
-  fprintf (_fp, "X-Mailer: NCSA Mosaic %s on %s\n", 
+  fprintf (_fp, "X-Mailer: mMosaic %s on %s\n", 
            MO_VERSION_STRING, MO_MACHINE_TYPE);
   if (url)
     fprintf (_fp, "X-URL: %s\n", url);
@@ -646,3 +643,78 @@ mo_status mo_send_mailto_message (char *text, char *to, char *subj,
 	mo_finish_sending_mailto_message ();
 	return mo_succeed;
 }
+
+/* Originally in whine.c and then in techsupport.c...now it's here. - SWP */
+/* ------------------------------------------------------------------------ */
+  
+FILE *mo_start_sending_mail_message (char *to, char *subj,
+                                     char *content_type, char *url)
+{ 
+        char cmd[2048];
+        char *tmp;
+        
+        if (!to)
+                return NULL;
+        
+        if(!strcmp(content_type,"url_only")){
+                content_type = "text/plain";
+        }
+/* Try listing address on command line. */
+        for (tmp = to; *tmp; tmp++)
+                if (*tmp == ',')
+                        *tmp = ' ';
+        
+        if (mMosaicAppData.mail_filter_command && content_type &&
+            strcmp (content_type, "application/postscript")) {
+                sprintf (cmd, "%s | %s %s", mMosaicAppData.mail_filter_command,
+                        mMosaicAppData.sendmail_command, to);
+        } else {
+                sprintf(cmd, "%s %s", mMosaicAppData.sendmail_command, to);
+        }
+        if ((_fp = popen (cmd, "w")) == NULL)
+                return NULL;
+        
+        fprintf (_fp, "To: %s\n", to);
+        fprintf (_fp, "Subject: %s\n", subj);
+        fprintf (_fp, "Reply-To: %s <%s>\n",mMosaicAppData.author_full_name,mMosaicAppData.author_email);
+        fprintf (_fp, "Content-Type: %s\n", content_type);
+        fprintf (_fp, "Mime-Version: 1.0\n");
+        fprintf (_fp, "X-Mailer: mMosaic %s on %s\n", MO_VERSION_STRING, MO_MACHINE_TYPE);
+        if (url)
+                fprintf (_fp, "X-URL: %s\n", url);
+        fprintf (_fp, "\n");
+/* Stick in BASE tag as appropriate. */
+        if (url && content_type && strcmp (content_type, "text/x-html") == 0)
+                fprintf (_fp, "<base href=\"%s\">\n", url);
+        return _fp;
+}       
+
+mo_status mo_finish_sending_mail_message (void)
+{ 
+        if (_fp)
+                pclose (_fp);
+        _fp = NULL;
+        return mo_succeed;
+} 
+/* ------------------------------------------------------------------------ */
+  
+mo_status mo_send_mail_message (char *text, char *to, char *subj,
+                                char *content_type, char *url)
+{ 
+        FILE *fp;
+        
+        fp = mo_start_sending_mail_message (to, subj, content_type, url);
+        if (!fp)
+                return mo_fail;
+        
+        if(!strcmp(content_type,"url_only")){
+                fputs(url,fp);
+                fputs("\n\n",fp);
+        } else {
+                fputs (text, fp);
+        }
+        mo_finish_sending_mail_message ();
+        return mo_succeed;
+}       
+ 
+
