@@ -1,31 +1,6 @@
 /* Please read copyright.ncsa. Don't remove next line */
-#include "copyright.ncsa"
-/*
- * The following XPM reading code was based on the libXpm code, which I
- * am free to use as long as I include the following copyright:
- */
-/*
- * Copyright 1990-93 GROUPE BULL
- *
- * Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of GROUPE BULL not be used in advertising
- * or publicity pertaining to distribution of the software without specific,
- * written prior permission.  GROUPE BULL makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
- *
- * GROUPE BULL disclaims all warranties with regard to this software,
- * including all implied warranties of merchantability and fitness,
- * in no event shall GROUPE BULL be liable for any special,
- * indirect or consequential damages or any damages
- * whatsoever resulting from loss of use, data or profits,
- * whether in an action of contract, negligence or other tortious
- * action, arising out of or in connection with the use 
- * or performance of this software.
- */
+#include "../Copyrights/copyright.ncsa"
+#include "../Copyrights/copyright.bull"
 
 #include <sys/time.h>
 struct timeval Tv;
@@ -41,16 +16,7 @@ struct timezone Tz;
 /*for memset*/
 #include <memory.h>
 
-extern Display *dsp;
-
-extern int installed_colormap;
-extern Colormap installed_cmap;
-
-#ifndef DISABLE_TRACE
-extern int srcTrace;
-#endif
-
-char *xpmColorKeys[] = {
+char *_MMxpmColorKeys[] = {
     "s",				/* key #1: symbol */
     "m",				/* key #2: mono visual */
     "g4",				/* key #3: 4 grays visual */
@@ -58,21 +24,15 @@ char *xpmColorKeys[] = {
     "c"					/* key #5: color visual */
 };
 
-xpmDataType xpmDataTypes[] = {
-    "", "!", "\n", '\0', '\n', "", "", "", "",	/* Natural type */
-    "C", "/*", "*/", '"', '"', ",\n", "static char *", "[] = {\n", "};\n",
-    "Lisp", ";", "\n", '"', '"', "\n", "(setq ", " '(\n", "))\n",
-#ifdef VMS
-    NULL
-#else
-    NULL, NULL, NULL, 0, 0, NULL, NULL, NULL, NULL
-#endif
+xpmDataType _MMxpmDataTypes[] = {
+    {"", "!", "\n", '\0', '\n', "", "", "", ""},	/* Natural type */
+    {"C", "/*", "*/", '"', '"', ",\n", "static char *", "[] = {\n", "};\n"},
+    {"Lisp", ";", "\n", '"', '"', "\n", "(setq ", " '(\n", "))\n"},
+    {NULL, NULL, NULL, 0, 0, NULL, NULL, NULL, NULL}
 };
 
-/*
- * Free the computed color table
- */
-void xpmFreeColorTable_( char ***colorTable, int ncolors)
+/* Free the computed color table */
+static void _MMxpmFreeColorTable_( char ***colorTable, int ncolors)
 {
     int a, b;
     char ***ct, **cts;
@@ -82,10 +42,10 @@ void xpmFreeColorTable_( char ***colorTable, int ncolors)
 	    if (*ct) {
 		for (b = 0, cts = *ct; b <= NKEYS; b++, cts++)
 		    if (*cts)
-			XpmFree(*cts);
-		XpmFree(*ct);
+			_MMXpmFree(*cts);
+		_MMXpmFree(*ct);
 	    }
-	XpmFree(colorTable);
+	_MMXpmFree(colorTable);
     }
 }
 
@@ -94,7 +54,7 @@ void xpmFreeColorTable_( char ***colorTable, int ncolors)
  * Intialize the xpmInternAttrib pointers to Null to know
  * which ones must be freed later on.
  */
-void xpmInitInternAttrib( xpmInternAttrib *attrib)
+void _MMxpmInitInternAttrib( xpmInternAttrib *attrib)
 {
     attrib->ncolors = 0;
     attrib->colorTable = NULL;
@@ -108,30 +68,30 @@ void xpmInitInternAttrib( xpmInternAttrib *attrib)
 /*
  * Free the xpmInternAttrib pointers which have been allocated
  */
-void xpmFreeInternAttrib( xpmInternAttrib *attrib)
+void _MMxpmFreeInternAttrib( xpmInternAttrib *attrib)
 {
     unsigned int a, ncolors;
     char **sptr;
 
     if (attrib->colorTable)
-	xpmFreeColorTable_(attrib->colorTable, attrib->ncolors);
+	_MMxpmFreeColorTable_(attrib->colorTable, attrib->ncolors);
     if (attrib->pixelindex)
-	XpmFree(attrib->pixelindex);
+	_MMXpmFree(attrib->pixelindex);
     if (attrib->xcolors)
-	XpmFree(attrib->xcolors);
+	_MMXpmFree(attrib->xcolors);
     if (attrib->colorStrings) {
 	ncolors = attrib->ncolors;
 	for (a = 0, sptr = attrib->colorStrings; a < ncolors; a++, sptr++)
 	    if (*sptr)
-		XpmFree(*sptr);
-	XpmFree(attrib->colorStrings);
+		_MMXpmFree(*sptr);
+	_MMXpmFree(attrib->colorStrings);
     }
 }
 
 /*
  * open the given file to be read as an xpmData which is returned.
  */
-int xpmReadFile( char *filename, xpmData *mdata)
+int _MMxpmReadFile( char *filename, xpmData *mdata)
 {
 #ifdef ZPIPE
     char *compressfile, buf[BUFSIZ];
@@ -166,7 +126,7 @@ int xpmReadFile( char *filename, xpmData *mdata)
 	    if (!stat(compressfile, &status)) {
 		sprintf(buf, "uncompress -c %s", compressfile);
 		if (!(mdata->stream.file = popen(buf, "r"))) {
-		    XpmFree(compressfile);
+		    _MMXpmFree(compressfile);
 		    return (XpmOpenFailed);
 		}
 		mdata->type = XPMPIPE;
@@ -176,7 +136,7 @@ int xpmReadFile( char *filename, xpmData *mdata)
 		if (!stat(compressfile, &status)) {
 		    sprintf(buf, "gunzip -c %s", compressfile);
 		    if (!(mdata->stream.file = popen(buf, "r"))) {
-			XpmFree(compressfile);
+			_MMXpmFree(compressfile);
 			return (XpmOpenFailed);
 		    }
 		    mdata->type = XPMPIPE;
@@ -184,7 +144,7 @@ int xpmReadFile( char *filename, xpmData *mdata)
 #endif
 		    if (!(mdata->stream.file = fopen(filename, "r"))) {
 #ifdef ZPIPE
-			XpmFree(compressfile);
+			_MMXpmFree(compressfile);
 #endif
 			return (XpmOpenFailed);
 		    }
@@ -192,7 +152,7 @@ int xpmReadFile( char *filename, xpmData *mdata)
 #ifdef ZPIPE
 		}
 	    }
-	    XpmFree(compressfile);
+	    _MMXpmFree(compressfile);
 	}
 #endif
     }
@@ -200,11 +160,8 @@ int xpmReadFile( char *filename, xpmData *mdata)
     return (XpmSuccess);
 }
 
-
-/*
- * close the file related to the xpmData if any
- */
-int xpmDataClose( xpmData *mdata)
+/* close the file related to the xpmData if any */
+int _MMxpmDataClose( xpmData *mdata)
 {
     switch (mdata->type) {
     case XPMARRAY:
@@ -336,7 +293,7 @@ static int ParseComment( xpmData *mdata)
 /*
  * skip to the end of the current string and the beginning of the next one
  */
-int xpmNextString( xpmData *mdata)
+int _MMxpmNextString( xpmData *mdata)
 {
     if (!mdata->type)
 	mdata->cptr = (mdata->stream.data)[++mdata->line];
@@ -386,7 +343,7 @@ int xpmNextString( xpmData *mdata)
     return 0;
 }
 
-unsigned int atoui( register char *p, unsigned int l, unsigned int *ui_return)
+static unsigned int atoui( register char *p, unsigned int l, unsigned int *ui_return)
 {
     register int n, i;
 
@@ -415,22 +372,22 @@ static int ParseValues( xpmData *data, unsigned int *width, unsigned int *height
     /*
      * read values: width, height, ncolors, chars_per_pixel
      */
-    if (!(xpmNextUI(data, width) && xpmNextUI(data, height)
-	  && xpmNextUI(data, ncolors) && xpmNextUI(data, cpp)))
+    if (!(_MMxpmNextUI(data, width) && _MMxpmNextUI(data, height)
+	  && _MMxpmNextUI(data, ncolors) && _MMxpmNextUI(data, cpp)))
 	return (XpmFileInvalid);
 
     /*
      * read optional information (hotspot and/or XPMEXT) if any
      */
-    l = xpmNextWord(data, buf);
+    l = _MMxpmNextWord(data, buf);
     if (l) {
 	*extensions = l == 6 && !strncmp("XPMEXT", buf, 6);
 	if (*extensions)
-	    *hotspot = xpmNextUI(data, x_hotspot)
-		&& xpmNextUI(data, y_hotspot);
+	    *hotspot = _MMxpmNextUI(data, x_hotspot)
+		&& _MMxpmNextUI(data, y_hotspot);
 	else {
-	    *hotspot = atoui(buf, l, x_hotspot) && xpmNextUI(data, y_hotspot);
-	    l = xpmNextWord(data, buf);
+	    *hotspot = atoui(buf, l, x_hotspot) && _MMxpmNextUI(data, y_hotspot);
+	    l = _MMxpmNextWord(data, buf);
 	    *extensions = l == 6 && !strncmp("XPMEXT", buf, 6);
 	}
     }
@@ -456,10 +413,10 @@ static int ParseColors( xpmData *data, unsigned int ncolors,
 	return (XpmNoMemory);
 
     for (a = 0, ct = colorTable; a < ncolors; a++, ct++) {
-	xpmNextString(data);		/* skip the line */
+	_MMxpmNextString(data);		/* skip the line */
 	cts = *ct = (char **) XpmCalloc((NKEYS + 1), sizeof(char *));
 	if (!cts) {
-	    xpmFreeColorTable_(colorTable, ncolors);
+	    _MMxpmFreeColorTable_(colorTable, ncolors);
 	    return (XpmNoMemory);
 	}
 
@@ -468,20 +425,20 @@ static int ParseColors( xpmData *data, unsigned int ncolors,
 	 */
 	*cts = (char *) XpmMalloc(cpp + 1);	/* + 1 for null terminated */
 	if (!*cts) {
-	    xpmFreeColorTable_(colorTable, ncolors);
+	    _MMxpmFreeColorTable_(colorTable, ncolors);
 	    return (XpmNoMemory);
 	}
 	for (b = 0, s = *cts; b < cpp; b++, s++)
-	    *s = xpmGetC(data);
+	    *s = _MMxpmGetC(data);
 	*s = '\0';
 
 	/*
 	 * store the string in the hashtable with its color index number
 	 */
 	if (USE_HASHTABLE) {
-	    ErrorStatus = xpmHashIntern(hashtable, *cts, HashAtomData(a));
+	    ErrorStatus = _MMxpmHashIntern(hashtable, *cts, HashAtomData(a));
 	    if (ErrorStatus != XpmSuccess) {
-		xpmFreeColorTable_(colorTable, ncolors);
+		_MMxpmFreeColorTable_(colorTable, ncolors);
 		return (ErrorStatus);
 	    }
 	}
@@ -491,9 +448,9 @@ static int ParseColors( xpmData *data, unsigned int ncolors,
 	 */
 	curkey = 0;
 	lastwaskey = 0;
-	while (l = xpmNextWord(data, buf)) {
+	while (l = _MMxpmNextWord(data, buf)) {
 	    if (!lastwaskey) {
-		for (key = 0, sptr = xpmColorKeys; key < NKEYS; key++, sptr++)
+		for (key = 0, sptr = _MMxpmColorKeys; key < NKEYS; key++, sptr++)
 		    if ((strlen(*sptr) == l) && (!strncmp(*sptr, buf, l)))
 			break;
 	    }
@@ -501,7 +458,7 @@ static int ParseColors( xpmData *data, unsigned int ncolors,
 		if (curkey) {		/* flush string */
 		    s = cts[curkey] = (char *) XpmMalloc(strlen(curbuf) + 1);
 		    if (!s) {
-			xpmFreeColorTable_(colorTable, ncolors);
+			_MMxpmFreeColorTable_(colorTable, ncolors);
 			return (XpmNoMemory);
 		    }
 		    strcpy(s, curbuf);
@@ -511,7 +468,7 @@ static int ParseColors( xpmData *data, unsigned int ncolors,
 		lastwaskey = 1;
 	    } else {
 		if (!curkey) {		/* key without value */
-		    xpmFreeColorTable_(colorTable, ncolors);
+		    _MMxpmFreeColorTable_(colorTable, ncolors);
 		    return (XpmFileInvalid);
 		}
 		if (!lastwaskey)
@@ -522,12 +479,12 @@ static int ParseColors( xpmData *data, unsigned int ncolors,
 	    }
 	}
 	if (!curkey) {			/* key without value */
-	    xpmFreeColorTable_(colorTable, ncolors);
+	    _MMxpmFreeColorTable_(colorTable, ncolors);
 	    return (XpmFileInvalid);
 	}
 	s = cts[curkey] = (char *) XpmMalloc(strlen(curbuf) + 1);
 	if (!s) {
-	    xpmFreeColorTable_(colorTable, ncolors);
+	    _MMxpmFreeColorTable_(colorTable, ncolors);
 	    return (XpmNoMemory);
 	}
 	strcpy(s, curbuf);
@@ -551,8 +508,7 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
     iptr = iptr2;
     switch (cpp) {
 
-    case (1):				/* Optimize for single character
-					 * colors */
+    case (1):			/* Optimize for single character colors */
 	{
 	    unsigned short colidx[256];
 
@@ -561,14 +517,14 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 		colidx[colorTable[a][0][0]] = a + 1;
 
 	    for (y = 0; y < height; y++) {
-		xpmNextString(data);
+		_MMxpmNextString(data);
 		for (x = 0; x < width; x++, iptr++) {
-		    int idx = colidx[xpmGetC(data)];
+		    int idx = colidx[_MMxpmGetC(data)];
 
 		    if (idx != 0)
 			*iptr = idx - 1;
 		    else {
-			XpmFree(iptr2);
+			_MMXpmFree(iptr2);
 			return (XpmFileInvalid);
 		    }
 		}
@@ -576,8 +532,7 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 	}
 	break;
 
-    case (2):				/* Optimize for double character
-					 * colors */
+    case (2):			/* Optimize for double character colors */
 	{
 	    unsigned short cidx[256][256];
 
@@ -586,15 +541,15 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 		cidx[colorTable[a][0][0]][colorTable[a][0][1]] = a + 1;
 
 	    for (y = 0; y < height; y++) {
-		xpmNextString(data);
+		_MMxpmNextString(data);
 		for (x = 0; x < width; x++, iptr++) {
-		    int cc1 = xpmGetC(data);
-		    int idx = cidx[cc1][xpmGetC(data)];
+		    int cc1 = _MMxpmGetC(data);
+		    int idx = cidx[cc1][_MMxpmGetC(data)];
 
 		    if (idx != 0)
 			*iptr = idx - 1;
 		    else {
-			XpmFree(iptr2);
+			_MMXpmFree(iptr2);
 			return (XpmFileInvalid);
 		    }
 		}
@@ -602,8 +557,7 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 	}
 	break;
 
-    default:				/* Non-optimized case of long color
-					 * names */
+    default:			/* Non-optimized case of long color names */
 	{
 	    char *s;
 	    char buf[BUFSIZ];
@@ -613,13 +567,13 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 		xpmHashAtom *slot;
 
 		for (y = 0; y < height; y++) {
-		    xpmNextString(data);
+		    _MMxpmNextString(data);
 		    for (x = 0; x < width; x++, iptr++) {
 			for (a = 0, s = buf; a < cpp; a++, s++)
-			    *s = xpmGetC(data);
-			slot = xpmHashSlot(hashtable, buf);
+			    *s = _MMxpmGetC(data);
+			slot = _MMxpmHashSlot(hashtable, buf);
 			if (!*slot) {	/* no color matches */
-			    XpmFree(iptr2);
+			    _MMXpmFree(iptr2);
 			    return (XpmFileInvalid);
 			}
 			*iptr = HashColorIndex(slot);
@@ -627,15 +581,15 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 		}
 	    } else {
 		for (y = 0; y < height; y++) {
-		    xpmNextString(data);
+		    _MMxpmNextString(data);
 		    for (x = 0; x < width; x++, iptr++) {
 			for (a = 0, s = buf; a < cpp; a++, s++)
-			    *s = xpmGetC(data);
+			    *s = _MMxpmGetC(data);
 			for (a = 0; a < ncolors; a++)
 			    if (!strcmp(colorTable[a][0], buf))
 				break;
 			if (a == ncolors) {	/* no color matches */
-			    XpmFree(iptr2);
+			    _MMXpmFree(iptr2);
 			    return (XpmFileInvalid);
 			}
 			*iptr = a;
@@ -653,7 +607,7 @@ static int ParsePixels( xpmData *data, unsigned int width, unsigned int height,
 /*
  * skip whitespace and return the following word
  */
-unsigned int xpmNextWord( xpmData *mdata, char *buf)
+unsigned int _MMxpmNextWord( xpmData *mdata, char *buf)
 {
     register unsigned int n = 0;
     int c;
@@ -690,12 +644,12 @@ unsigned int xpmNextWord( xpmData *mdata, char *buf)
  * skip whitespace and compute the following unsigned int,
  * returns 1 if one is found and 0 if not
  */
-int xpmNextUI( xpmData *mdata, unsigned int *ui_return)
+int _MMxpmNextUI( xpmData *mdata, unsigned int *ui_return)
 {
     char buf[BUFSIZ];
     int l;
 
-    l = xpmNextWord(mdata, buf);
+    l = _MMxpmNextWord(mdata, buf);
     return atoui(buf, l, ui_return);
 }
 
@@ -703,7 +657,7 @@ int xpmNextUI( xpmData *mdata, unsigned int *ui_return)
 /*
  * parse xpm header
  */
-int xpmParseHeader( xpmData *mdata)
+int _MMxpmParseHeader( xpmData *mdata)
 {
     char buf[BUFSIZ];
     int l, n = 0;
@@ -712,39 +666,39 @@ int xpmParseHeader( xpmData *mdata)
 	mdata->Bos = '\0';
 	mdata->Eos = '\n';
 	mdata->Bcmt = mdata->Ecmt = NULL;
-	xpmNextWord(mdata, buf);	/* skip the first word */
-	l = xpmNextWord(mdata, buf);	/* then get the second word */
+	_MMxpmNextWord(mdata, buf);	/* skip the first word */
+	l = _MMxpmNextWord(mdata, buf);	/* then get the second word */
 	if ((l == 3 && !strncmp("XPM", buf, 3)) ||
 	    (l == 4 && !strncmp("XPM2", buf, 4))) {
 	    if (l == 3)
 		n = 1;			/* handle XPM as XPM2 C */
 	    else {
-		l = xpmNextWord(mdata, buf);	/* get the type key word */
+		l = _MMxpmNextWord(mdata, buf);	/* get the type key word */
 
 		/*
 		 * get infos about this type
 		 */
-		while (xpmDataTypes[n].type
-		       && strncmp(xpmDataTypes[n].type, buf, l))
+		while (_MMxpmDataTypes[n].type
+		       && strncmp(_MMxpmDataTypes[n].type, buf, l))
 		    n++;
 	    }
-	    if (xpmDataTypes[n].type) {
+	    if (_MMxpmDataTypes[n].type) {
 		if (n == 0) {		/* natural type */
-		    mdata->Bcmt = xpmDataTypes[n].Bcmt;
-		    mdata->Ecmt = xpmDataTypes[n].Ecmt;
-		    xpmNextString(mdata);	/* skip the end of
+		    mdata->Bcmt = _MMxpmDataTypes[n].Bcmt;
+		    mdata->Ecmt = _MMxpmDataTypes[n].Ecmt;
+		    _MMxpmNextString(mdata);	/* skip the end of
 						 * headerline */
-		    mdata->Bos = xpmDataTypes[n].Bos;
+		    mdata->Bos = _MMxpmDataTypes[n].Bos;
 		} else {
-		    xpmNextString(mdata);	/* skip the end of
+		    _MMxpmNextString(mdata);	/* skip the end of
 						 * headerline */
-		    mdata->Bcmt = xpmDataTypes[n].Bcmt;
-		    mdata->Ecmt = xpmDataTypes[n].Ecmt;
-		    mdata->Bos = xpmDataTypes[n].Bos;
+		    mdata->Bcmt = _MMxpmDataTypes[n].Bcmt;
+		    mdata->Ecmt = _MMxpmDataTypes[n].Ecmt;
+		    mdata->Bos = _MMxpmDataTypes[n].Bos;
 		    mdata->Eos = '\0';
-		    xpmNextString(mdata);	/* skip the assignment line */
+		    _MMxpmNextString(mdata);	/* skip the assignment line */
 		}
-		mdata->Eos = xpmDataTypes[n].Eos;
+		mdata->Eos = _MMxpmDataTypes[n].Eos;
 	    } else
 		return XpmFileInvalid;
 	} else
@@ -757,7 +711,7 @@ int xpmParseHeader( xpmData *mdata)
 /*
  * get the current comment line
  */
-int xpmGetCmt( xpmData *mdata, char **cmt)
+int _MMxpmGetCmt( xpmData *mdata, char **cmt)
 {
     if (!mdata->type)
 	*cmt = NULL;
@@ -774,18 +728,18 @@ int xpmGetCmt( xpmData *mdata, char **cmt)
 
 #undef RETURN
 #define RETURN(status) \
-  { if (colorTable) xpmFreeColorTable_(colorTable, ncolors); \
-    if (pixelindex) XpmFree(pixelindex); \
-    if (hints_cmt)  XpmFree(hints_cmt); \
-    if (colors_cmt) XpmFree(colors_cmt); \
-    if (pixels_cmt) XpmFree(pixels_cmt); \
+  { if (colorTable) _MMxpmFreeColorTable_(colorTable, ncolors); \
+    if (pixelindex) _MMXpmFree(pixelindex); \
+    if (hints_cmt)  _MMXpmFree(hints_cmt); \
+    if (colors_cmt) _MMXpmFree(colors_cmt); \
+    if (pixels_cmt) _MMXpmFree(pixels_cmt); \
     return(status); }
 
 /*
  * This function parses an Xpm file or data and store the found informations
  * in an an xpmInternAttrib structure which is returned.
  */
-int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
+int _MMxpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
     XpmAttributes *attributes)
 {
     /* variables to return */
@@ -803,7 +757,7 @@ int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
     /*
      * parse the header
      */
-    ErrorStatus = xpmParseHeader(data);
+    ErrorStatus = _MMxpmParseHeader(data);
     if (ErrorStatus != XpmSuccess)
 	return (ErrorStatus);
 
@@ -819,13 +773,13 @@ int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
      * store the hints comment line
      */
     if (attributes && (attributes->valuemask & XpmReturnInfos))
-	xpmGetCmt(data, &hints_cmt);
+	_MMxpmGetCmt(data, &hints_cmt);
 
     /*
      * init the hastable
      */
     if (USE_HASHTABLE) {
-	ErrorStatus = xpmHashTableInit(&hashtable);
+	ErrorStatus = _MMxpmHashTableInit(&hashtable);
 	if (ErrorStatus != XpmSuccess)
 	    return (ErrorStatus);
     }
@@ -841,7 +795,7 @@ int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
      * store the colors comment line
      */
     if (attributes && (attributes->valuemask & XpmReturnInfos))
-	xpmGetCmt(data, &colors_cmt);
+	_MMxpmGetCmt(data, &colors_cmt);
 
     /*
      * read pixels and index them on color number
@@ -853,7 +807,7 @@ int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
      * free the hastable
      */
     if (USE_HASHTABLE)
-	xpmHashTableFree(&hashtable);
+	_MMxpmHashTableFree(&hashtable);
 
     if (ErrorStatus != XpmSuccess)
 	RETURN(ErrorStatus);
@@ -862,7 +816,7 @@ int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
      * store the pixels comment line
      */
     if (attributes && (attributes->valuemask & XpmReturnInfos))
-	xpmGetCmt(data, &pixels_cmt);
+	_MMxpmGetCmt(data, &pixels_cmt);
 
     /*
      * store found informations in the xpmInternAttrib structure
@@ -893,7 +847,7 @@ int xpmParseData( xpmData *data, xpmInternAttrib *attrib_return,
 /*
  * open the given array to be read or written as an xpmData which is returned
  */
-void xpmOpenArray( char **data, xpmData *mdata)
+void _MMxpmOpenArray( char **data, xpmData *mdata)
 {
     mdata->type = XPMARRAY;
     mdata->stream.data = data;
@@ -904,7 +858,7 @@ void xpmOpenArray( char **data, xpmData *mdata)
     mdata->Bos = mdata->Eos = '\0';
 }
 
-unsigned char *ReadXpm3Pixmap( FILE *fp, char *datafile, int *w, int *h,
+unsigned char *_MMReadXpm3Pixmap( Widget view, FILE *fp, char *datafile, int *w, int *h,
 	XColor *colrs, int *bg)
 {
 	xpmData mdata;
@@ -919,29 +873,22 @@ unsigned char *ReadXpm3Pixmap( FILE *fp, char *datafile, int *w, int *h,
 	unsigned char *pix_data;
 	unsigned char *bptr;
 	unsigned int *pixels;
-        extern Widget view;
 
-#ifndef DISABLE_TRACE
-	if (srcTrace) {
-		gettimeofday(&Tv, &Tz);
-		fprintf(stderr, "ReadXpm3Pixmap enter (%d.%d)\n", Tv.tv_sec, Tv.tv_usec);
-	}
-#endif
 	*w = 0;
 	*h = 0;
 
 	attributes.valuemask = XpmReturnPixels;
 
-	if ((ErrorStatus = xpmReadFile(datafile, &mdata)) != XpmSuccess) {
+	if ((ErrorStatus = _MMxpmReadFile(datafile, &mdata)) != XpmSuccess) {
 		return(NULL);
 	}
 
-	xpmInitInternAttrib(&attrib);
+	_MMxpmInitInternAttrib(&attrib);
 
-	ErrorStatus = xpmParseData(&mdata, &attrib, &attributes);
+	ErrorStatus = _MMxpmParseData(&mdata, &attrib, &attributes);
 	if (ErrorStatus != XpmSuccess) {
-		xpmFreeInternAttrib(&attrib);
-		xpmDataClose(&mdata);
+		_MMxpmFreeInternAttrib(&attrib);
+		_MMxpmDataClose(&mdata);
 		return(NULL);
 	}
 
@@ -962,19 +909,10 @@ unsigned char *ReadXpm3Pixmap( FILE *fp, char *datafile, int *w, int *h,
 			tmpcolr.pixel = bg_pixel;
 
 			/* Now query for the full color info. */
-			XQueryColor (XtDisplay (view), 
-				(installed_colormap ?
-				 installed_cmap :
-				 DefaultColormap (XtDisplay (view),
-						  DefaultScreen (XtDisplay (view)))),
-			       &tmpcolr);
+			HTMLPixelToXColor(&tmpcolr);
 			*bg = i;
 		} else {
-			XParseColor(dsp,
-				(installed_colormap ?
-				 installed_cmap :
-				 DefaultColormap(dsp, DefaultScreen(dsp))),
-				colorName, &tmpcolr);
+			XParseColor(mMosaicDisplay, mMosaicColormap, colorName, &tmpcolr);
 		}
 		colrs[i].red = tmpcolr.red;
 		colrs[i].green = tmpcolr.green;
@@ -993,14 +931,10 @@ unsigned char *ReadXpm3Pixmap( FILE *fp, char *datafile, int *w, int *h,
 	pixels = attrib.pixelindex;
 	pix_data = (unsigned char *)malloc((*w) * (*h));
         if (pix_data == NULL) {
-#ifndef DISABLE_TRACE
-		if (srcTrace) {
-			fprintf(stderr, "Not enough memory for data.\n");
-		}
-#endif
+		fprintf(stderr, "Not enough memory for data.\n");
 
-		xpmFreeInternAttrib(&attrib);
-		xpmDataClose(&mdata);
+		_MMxpmFreeInternAttrib(&attrib);
+		_MMxpmDataClose(&mdata);
                 return((unsigned char *)NULL);
         }
 	bptr = pix_data;
@@ -1014,20 +948,13 @@ unsigned char *ReadXpm3Pixmap( FILE *fp, char *datafile, int *w, int *h,
                 pixels++;
         }
 
-	xpmFreeInternAttrib(&attrib);
-	xpmDataClose(&mdata);
-
-#ifndef DISABLE_TRACE
-	if (srcTrace) {
-		gettimeofday(&Tv, &Tz);
-		fprintf(stderr, "ReadXpm3Pixmap exit (%d.%d)\n", Tv.tv_sec, Tv.tv_usec);
-	}
-#endif
+	_MMxpmFreeInternAttrib(&attrib);
+	_MMxpmDataClose(&mdata);
 
         return(pix_data);
 }
 
-unsigned char *ProcessXpm3Data( Widget wid, char **xpmdata,
+unsigned char *_MMProcessXpm3Data( Widget wid, char **xpmdata,
 	int *w, int *h, XColor *colrs, int *bg)
 {
 	xpmData mdata;
@@ -1042,17 +969,16 @@ unsigned char *ProcessXpm3Data( Widget wid, char **xpmdata,
 	unsigned char *pix_data;
 	unsigned char *bptr;
 	unsigned int *pixels;
-        extern Widget view;
 
 	*w = 0;
 	*h = 0;
 	attributes.valuemask = XpmReturnPixels;
-	xpmOpenArray(xpmdata, &mdata);
-	xpmInitInternAttrib(&attrib);
-	ErrorStatus = xpmParseData(&mdata, &attrib, &attributes);
+	_MMxpmOpenArray(xpmdata, &mdata);
+	_MMxpmInitInternAttrib(&attrib);
+	ErrorStatus = _MMxpmParseData(&mdata, &attrib, &attributes);
 	if (ErrorStatus != XpmSuccess) {
-		xpmFreeInternAttrib(&attrib);
-		xpmDataClose(&mdata);
+		_MMxpmFreeInternAttrib(&attrib);
+		_MMxpmDataClose(&mdata);
 		return(NULL);
 	}
 
@@ -1073,19 +999,10 @@ unsigned char *ProcessXpm3Data( Widget wid, char **xpmdata,
 			tmpcolr.pixel = bg_pixel;
 
 			/* Now query for the full color info. */
-			XQueryColor (XtDisplay (wid), 
-				(installed_colormap ?
-				 installed_cmap :
-				 DefaultColormap (XtDisplay (wid),
-						  DefaultScreen (XtDisplay (wid)))),
-			       &tmpcolr);
+			HTMLPixelToXColor(&tmpcolr);
 			*bg = i;
 		} else {
-			XParseColor(XtDisplay (wid),
-				(installed_colormap ?
-				 installed_cmap :
-				 DefaultColormap(XtDisplay (wid),
-						 DefaultScreen(XtDisplay (wid)))),
+			XParseColor(XtDisplay(wid), mMosaicColormap,
 				colorName, &tmpcolr);
 		}
 		colrs[i].red = tmpcolr.red;
@@ -1105,14 +1022,9 @@ unsigned char *ProcessXpm3Data( Widget wid, char **xpmdata,
 	pixels = attrib.pixelindex;
 	pix_data = (unsigned char *)malloc((*w) * (*h));
         if (pix_data == NULL) {
-#ifndef DISABLE_TRACE
-		if (srcTrace) {
-			fprintf(stderr, "Not enough memory for data.\n");
-		}
-#endif
-
-		xpmFreeInternAttrib(&attrib);
-		xpmDataClose(&mdata);
+		fprintf(stderr, "Not enough memory for data.\n");
+		_MMxpmFreeInternAttrib(&attrib);
+		_MMxpmDataClose(&mdata);
                 return((unsigned char *)NULL);
         }
 	bptr = pix_data;
@@ -1126,15 +1038,8 @@ unsigned char *ProcessXpm3Data( Widget wid, char **xpmdata,
                 pixels++;
         }
 
-	xpmFreeInternAttrib(&attrib);
-	xpmDataClose(&mdata);
-
-#ifndef DISABLE_TRACE
-	if (srcTrace) {
-		gettimeofday(&Tv, &Tz);
-		fprintf(stderr, "ReadXpm3Pixmap exit (%d.%d)\n", Tv.tv_sec, Tv.tv_usec);
-	}
-#endif
+	_MMxpmFreeInternAttrib(&attrib);
+	_MMxpmDataClose(&mdata);
 
         return(pix_data);
 }
